@@ -23,6 +23,9 @@
 
 #include <limits>
 
+#include "gettext.h"
+#define _(string) gettext(string)
+
 #include "audio_output.h"
 #include "lib_versions.h"
 
@@ -57,12 +60,12 @@ void audio_output::init()
     {
         if (!(_device = alcOpenDevice(NULL)))
         {
-            throw exc("No OpenAL device available.");
+            throw exc(_("No OpenAL device available."));
         }
         if (!(_context = alcCreateContext(_device, NULL)))
         {
             alcCloseDevice(_device);
-            throw exc("No OpenAL context available.");
+            throw exc(_("No OpenAL context available."));
         }
         alcMakeContextCurrent(_context);
         set_openal_versions();
@@ -73,7 +76,7 @@ void audio_output::init()
             alcMakeContextCurrent(NULL);
             alcDestroyContext(_context);
             alcCloseDevice(_device);
-            throw exc("Cannot create OpenAL buffers.");
+            throw exc(_("Cannot create OpenAL buffers."));
         }
         alGenSources(1, &_source);
         if (alGetError() != AL_NO_ERROR)
@@ -82,7 +85,7 @@ void audio_output::init()
             alcMakeContextCurrent(NULL);
             alcDestroyContext(_context);
             alcCloseDevice(_device);
-            throw exc("Cannot create OpenAL source.");
+            throw exc(_("Cannot create OpenAL source."));
         }
         /* Comment from alffmpeg.c:
          * "Set parameters so mono sources won't distance attenuate" */
@@ -95,7 +98,7 @@ void audio_output::init()
             alcMakeContextCurrent(NULL);
             alcDestroyContext(_context);
             alcCloseDevice(_device);
-            throw exc("Cannot set OpenAL source parameters.");
+            throw exc(_("Cannot set OpenAL source parameters."));
         }
         _state = 0;
         _initialized = true;
@@ -149,14 +152,14 @@ int64_t audio_output::status(bool *need_data)
             alGetSourcei(_source, AL_SOURCE_STATE, &_state);
             if (alGetError() != AL_NO_ERROR)
             {
-                throw exc("Cannot check OpenAL source state.");
+                throw exc(_("Cannot check OpenAL source state."));
             }
             if (_state != AL_PLAYING)
             {
                 alSourcePlay(_source);
                 if (alGetError() != AL_NO_ERROR)
                 {
-                    throw exc("Cannot restart OpenAL source playback.");
+                    throw exc(_("Cannot restart OpenAL source playback."));
                 }
             }
             if (need_data)
@@ -309,8 +312,8 @@ ALenum audio_output::get_al_format(const audio_blob &blob)
     }
     if (format == 0)
     {
-        throw exc(std::string("No OpenAL format available for audio data ")
-                + "(" + blob.format_name() + ").");
+        throw exc(str::asprintf(_("No OpenAL format available for "
+                        "audio data format %s."), blob.format_name().c_str()));
     }
     return format;
 }
@@ -336,7 +339,7 @@ void audio_output::data(const audio_blob &blob)
         }
         if (alGetError() != AL_NO_ERROR)
         {
-            throw exc("Cannot buffer initial OpenAL data.");
+            throw exc(_("Cannot buffer initial OpenAL data."));
         }
     }
     else if (blob.size > 0)
@@ -350,7 +353,7 @@ void audio_output::data(const audio_blob &blob)
         alSourceQueueBuffers(_source, 1, &buf);
         if (alGetError() != AL_NO_ERROR)
         {
-            throw exc("Cannot buffer OpenAL data.");
+            throw exc(_("Cannot buffer OpenAL data."));
         }
         // Update the time spent on all past buffers
         int64_t current_buffer_samples = _buffer_size / _buffer_channels[0] * 8 / _buffer_sample_bits[0];
@@ -375,7 +378,7 @@ int64_t audio_output::start()
     alGetSourcei(_source, AL_SOURCE_STATE, &_state);
     if (alGetError() != AL_NO_ERROR)
     {
-        throw exc("Cannot start OpenAL source playback.");
+        throw exc(_("Cannot start OpenAL source playback."));
     }
     _past_time = 0;
     _last_timestamp = 0;
@@ -389,7 +392,7 @@ void audio_output::pause()
     alSourcePause(_source);
     if (alGetError() != AL_NO_ERROR)
     {
-        throw exc("Cannot pause OpenAL source playback.");
+        throw exc(_("Cannot pause OpenAL source playback."));
     }
 }
 
@@ -398,7 +401,7 @@ void audio_output::unpause()
     alSourcePlay(_source);
     if (alGetError() != AL_NO_ERROR)
     {
-        throw exc("Cannot unpause OpenAL source playback.");
+        throw exc(_("Cannot unpause OpenAL source playback."));
     }
 }
 
@@ -407,7 +410,7 @@ void audio_output::stop()
     alSourceStop(_source);
     if (alGetError() != AL_NO_ERROR)
     {
-        throw exc("Cannot stop OpenAL source playback.");
+        throw exc(_("Cannot stop OpenAL source playback."));
     }
     // flush all buffers and reset the state
     ALint processed_buffers;
@@ -418,7 +421,7 @@ void audio_output::stop()
         alSourceUnqueueBuffers(_source, 1, &buf);
         if (alGetError() != AL_NO_ERROR)
         {
-            throw exc("Cannot unqueue OpenAL source buffers.");
+            throw exc(_("Cannot unqueue OpenAL source buffers."));
         }
         alGetSourcei(_source, AL_BUFFERS_PROCESSED, &processed_buffers);
     }
