@@ -3,6 +3,7 @@
  *
  * Copyright (C) 2010-2011
  * Martin Lambers <marlam@marlam.de>
+ * Joe <joe@wpj.cz>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,6 +25,9 @@
 #include <cstring>
 #include <cmath>
 
+#include "gettext.h"
+#define _(string) gettext(string)
+
 #include "media_data.h"
 
 #include "str.h"
@@ -255,7 +259,8 @@ std::string video_frame::format_name() const
 
 std::string video_frame::format_info() const
 {
-    return str::asprintf("%dx%d, %.3g:1", raw_width, raw_height, aspect_ratio);
+    /* TRANSLATORS: This is a very short string describing the video size and aspect ratio. */
+    return str::asprintf(_("%dx%d, %.3g:1"), raw_width, raw_height, aspect_ratio);
 }
 
 static int next_multiple_of_4(int x)
@@ -370,6 +375,7 @@ void video_frame::copy_plane(int view, int plane, void *buf) const
 }
 
 audio_blob::audio_blob() :
+    language(),
     channels(-1),
     rate(-1),
     sample_format(u8),
@@ -381,7 +387,10 @@ audio_blob::audio_blob() :
 
 std::string audio_blob::format_info() const
 {
-    return str::asprintf("%d channels, %g kHz, %d bit", channels, rate / 1e3f, sample_bits());
+    /* TRANSLATORS: This is a very short string describing the audio language, channels, frequency, and bits. */
+    return str::asprintf(_("%s, %d ch., %g kHz, %d bit"),
+            language.empty() ? _("unknown") : language.c_str(),
+            channels, rate / 1e3f, sample_bits());
 }
 
 std::string audio_blob::format_name() const
@@ -402,7 +411,9 @@ std::string audio_blob::format_name() const
         sample_format_name = "d64";
         break;
     }
-    return str::asprintf("%d-%d-%s", channels, rate, sample_format_name);
+    return str::asprintf("%s-%d-%d-%s",
+            language.empty() ? _("unknown") : language.c_str(),
+            channels, rate, sample_format_name);
 }
 
 int audio_blob::sample_bits() const
@@ -426,6 +437,92 @@ int audio_blob::sample_bits() const
     return bits;
 }
 
+subtitle_box::subtitle_box() :
+    language(),
+    format(text),
+    style(),
+    str(),
+    images(),
+    presentation_start_time(std::numeric_limits<int64_t>::min()),
+    presentation_stop_time(std::numeric_limits<int64_t>::min())
+{
+}
+
+std::string subtitle_box::format_info() const
+{
+    return (language.empty() ? _("unknown") : language);
+}
+
+std::string subtitle_box::format_name() const
+{
+    return (language.empty() ? _("unknown") : language);
+}
+
+void subtitle_box::image_t::save(std::ostream &os) const
+{
+    s11n::save(os, w);
+    s11n::save(os, h);
+    s11n::save(os, x);
+    s11n::save(os, y);
+    s11n::save(os, palette.size());
+    if (palette.size() > 0)
+    {
+        s11n::save(os, static_cast<const void *>(&(palette[0])), palette.size() * sizeof(uint8_t));
+    }
+    s11n::save(os, data.size());
+    if (data.size() > 0)
+    {
+        s11n::save(os, static_cast<const void *>(&(data[0])), data.size() * sizeof(uint8_t));
+    }
+    s11n::save(os, linesize);
+}
+
+void subtitle_box::image_t::load(std::istream &is)
+{
+    size_t s;
+    s11n::load(is, w);
+    s11n::load(is, h);
+    s11n::load(is, x);
+    s11n::load(is, y);
+    s11n::load(is, s);
+    palette.resize(s);
+    if (palette.size() > 0)
+    {
+        s11n::load(is, static_cast<void *>(&(palette[0])), palette.size() * sizeof(uint8_t));
+    }
+    s11n::load(is, s);
+    data.resize(s);
+    if (data.size() > 0)
+    {
+        s11n::load(is, static_cast<void *>(&(data[0])), data.size() * sizeof(uint8_t));
+    }
+    s11n::load(is, linesize);
+}
+
+void subtitle_box::save(std::ostream &os) const
+{
+    s11n::save(os, language);
+    s11n::save(os, static_cast<int>(format));
+    s11n::save(os, style);
+    s11n::save(os, str);
+    s11n::save(os, images);
+    s11n::save(os, presentation_start_time);
+    s11n::save(os, presentation_stop_time);
+}
+
+void subtitle_box::load(std::istream &is)
+{
+    s11n::load(is, language);
+    int x;
+    s11n::load(is, x);
+    format = static_cast<format_t>(x);
+    s11n::load(is, style);
+    s11n::load(is, str);
+    s11n::load(is, images);
+    s11n::load(is, presentation_start_time);
+    s11n::load(is, presentation_stop_time);
+}
+
 parameters::parameters() :
     stereo_mode(stereo),
     stereo_mode_swap(false),
@@ -437,7 +534,13 @@ parameters::parameters() :
     contrast(std::numeric_limits<float>::quiet_NaN()),
     brightness(std::numeric_limits<float>::quiet_NaN()),
     hue(std::numeric_limits<float>::quiet_NaN()),
-    saturation(std::numeric_limits<float>::quiet_NaN())
+    saturation(std::numeric_limits<float>::quiet_NaN()),
+    subtitle_encoding(1, '\0'),
+    subtitle_font(1, '\0'),
+    subtitle_size(std::numeric_limits<int>::min()),
+    subtitle_scale(-1.0f),
+    subtitle_color(std::numeric_limits<uint64_t>::max()),
+    subtitle_parallax(std::numeric_limits<float>::quiet_NaN())
 {
 }
 
@@ -479,6 +582,30 @@ void parameters::set_defaults()
     {
         saturation = 0.0f;
     }
+    if (subtitle_encoding.length() == 1 && subtitle_encoding[0] == '\0')
+    {
+        subtitle_encoding = "";
+    }
+    if (subtitle_font.length() == 1 && subtitle_font[0] == '\0')
+    {
+        subtitle_font = "";
+    }
+    if (subtitle_size < -1)
+    {
+        subtitle_size = -1;
+    }
+    if (!std::isfinite(subtitle_scale) || subtitle_scale < 0.0f)
+    {
+        subtitle_scale = -1.0f;
+    }
+    if (subtitle_color > std::numeric_limits<uint32_t>::max())
+    {
+        subtitle_color = std::numeric_limits<uint64_t>::max();
+    }
+    if (!std::isfinite(subtitle_parallax) || subtitle_parallax < -1.0f || subtitle_parallax > +1.0f)
+    {
+        subtitle_parallax = 0.0f;
+    }
 }
 
 std::string parameters::stereo_mode_to_string(stereo_mode_t stereo_mode, bool stereo_mode_swap)
@@ -516,6 +643,9 @@ std::string parameters::stereo_mode_to_string(stereo_mode_t stereo_mode, bool st
     case checkerboard:
         s = "checkerboard";
         break;
+    case hdmi_frame_pack:
+        s = "hdmi-frame-pack";
+        break;
     case red_cyan_monochrome:
         s = "red-cyan-monochrome";
         break;
@@ -620,6 +750,10 @@ void parameters::stereo_mode_from_string(const std::string &s, stereo_mode_t &st
     {
         stereo_mode = checkerboard;
     }
+    else if (t == "hdmi-frame-pack")
+    {
+        stereo_mode = hdmi_frame_pack;
+    }
     else if (t == "red-cyan-monochrome")
     {
         stereo_mode = red_cyan_monochrome;
@@ -696,6 +830,12 @@ void parameters::save(std::ostream &os) const
     s11n::save(os, brightness);
     s11n::save(os, hue);
     s11n::save(os, saturation);
+    s11n::save(os, subtitle_encoding);
+    s11n::save(os, subtitle_font);
+    s11n::save(os, subtitle_size);
+    s11n::save(os, subtitle_scale);
+    s11n::save(os, subtitle_color);
+    s11n::save(os, subtitle_parallax);
 }
 
 void parameters::load(std::istream &is)
@@ -713,4 +853,10 @@ void parameters::load(std::istream &is)
     s11n::load(is, brightness);
     s11n::load(is, hue);
     s11n::load(is, saturation);
+    s11n::load(is, subtitle_encoding);
+    s11n::load(is, subtitle_font);
+    s11n::load(is, subtitle_size);
+    s11n::load(is, subtitle_scale);
+    s11n::load(is, subtitle_color);
+    s11n::load(is, subtitle_parallax);
 }
