@@ -4,6 +4,7 @@
  * Copyright (C) 2010-2011
  * Martin Lambers <marlam@marlam.de>
  * Frédéric Devernay <frederic.devernay@inrialpes.fr>
+ * Joe <cuchac@email.cz>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,6 +24,9 @@
 
 #include <limits>
 
+#include "gettext.h"
+#define _(string) gettext(string)
+
 #include "dbg.h"
 #include "exc.h"
 #include "msg.h"
@@ -32,9 +36,9 @@
 
 
 media_input::media_input() :
-    _active_video_stream(-1), _active_audio_stream(-1),
-    _have_active_video_read(false), _have_active_audio_read(false), _last_audio_data_size(0),
-    _initial_skip(0), _duration(-1)
+    _active_video_stream(-1), _active_audio_stream(-1), _active_subtitle_stream(-1),
+    _have_active_video_read(false), _have_active_audio_read(false), _have_active_subtitle_read(false),
+    _last_audio_data_size(0), _initial_skip(0), _duration(-1)
 {
 }
 
@@ -76,6 +80,23 @@ void media_input::get_audio_stream(int stream, int &media_object, int &media_obj
     }
 }
 
+void media_input::get_subtitle_stream(int stream, int &media_object, int &media_object_subtitle_stream) const
+{
+    assert(stream < subtitle_streams());
+
+    for (size_t i = 0; i < _media_objects.size(); i++)
+    {
+        if (_media_objects[i].subtitle_streams() < stream + 1)
+        {
+            stream -= _media_objects[i].subtitle_streams();
+            continue;
+        }
+        media_object = i;
+        media_object_subtitle_stream = stream;
+        break;
+    }
+}
+
 // Get the basename of an URL (just the file name, without leading paths)
 static std::string basename(const std::string &url)
 {
@@ -118,33 +139,61 @@ void media_input::open(const std::vector<std::string> &urls)
     // Gather metadata
     for (size_t i = 0; i < _media_objects.size(); i++)
     {
-        std::string pfx = (_media_objects.size() == 1 ? "" : str::from(i + 1) + " - ");
+        // Note that we may have multiple identical tag names in our metadata
         for (size_t j = 0; j < _media_objects[i].tags(); j++)
         {
-            _tag_names.push_back(pfx + _media_objects[i].tag_name(j));
-            _tag_values.push_back(pfx + _media_objects[i].tag_value(j));
+            _tag_names.push_back(_media_objects[i].tag_name(j));
+            _tag_values.push_back(_media_objects[i].tag_value(j));
         }
     }
 
     // Gather streams and stream names
     for (size_t i = 0; i < _media_objects.size(); i++)
     {
-        std::string pfx = (_media_objects.size() == 1 ? "" : str::from(i + 1) + " - ");
         for (int j = 0; j < _media_objects[i].video_streams(); j++)
         {
-            std::string pfx2 = (_media_objects[i].video_streams() == 1 ? "" : str::from(j + 1) + " - ");
-            _video_stream_names.push_back(pfx + pfx2
-                    + _media_objects[i].video_frame_template(j).format_info());
+            _video_stream_names.push_back(_media_objects[i].video_frame_template(j).format_info());
+        }
+    }
+    if (_video_stream_names.size() > 1)
+    {
+        for (size_t i = 0; i < _video_stream_names.size(); i++)
+        {
+            _video_stream_names[i].insert(0,
+                    std::string(1, '#') + str::from(i + 1) + '/'
+                    + str::from(_video_stream_names.size()) + ": ");
         }
     }
     for (size_t i = 0; i < _media_objects.size(); i++)
     {
-        std::string pfx = (_media_objects.size() == 1 ? "" : str::from(i + 1) + " - ");
         for (int j = 0; j < _media_objects[i].audio_streams(); j++)
         {
-            std::string pfx2 = (_media_objects[i].audio_streams() == 1 ? "" : str::from(j + 1) + " - ");
-            _audio_stream_names.push_back(pfx + pfx2
-                    + _media_objects[i].audio_blob_template(j).format_info());
+            _audio_stream_names.push_back(_media_objects[i].audio_blob_template(j).format_info());
+        }
+    }
+    if (_audio_stream_names.size() > 1)
+    {
+        for (size_t i = 0; i < _audio_stream_names.size(); i++)
+        {
+            _audio_stream_names[i].insert(0,
+                    std::string(1, '#') + str::from(i + 1) + '/'
+                    + str::from(_audio_stream_names.size()) + ": ");
+        }
+    }
+    for (size_t i = 0; i < _media_objects.size(); i++)
+    {
+        for (int j = 0; j < _media_objects[i].subtitle_streams(); j++)
+        {
+            _subtitle_stream_names.push_back(_media_objects[i].subtitle_box_template(j).format_info());
+        }
+    }
+    if (_subtitle_stream_names.size() > 1)
+    {
+        for (size_t i = 0; i < _subtitle_stream_names.size(); i++)
+        {
+            _subtitle_stream_names[i].insert(0,
+                    std::string(1, '#') + str::from(i + 1) + '/'
+                    + str::from(_subtitle_stream_names.size()) + ": ");
         }
     }
 
@@ -168,8 +217,9 @@ void media_input::open(const std::vector<std::string> &urls)
                 _duration = d;
             }
         }
+        // Ignore subtitle stream duration; it seems unreliable and is not important anyway.
     }
- 
+
     // Skip advertisement in 3dtv.at movies. Only works for single media objects.
     try { _initial_skip = str::to<int64_t>(tag_value("StereoscopicSkip")); } catch (...) { }
 
@@ -227,34 +277,48 @@ void media_input::open(const std::vector<std::string> &urls)
         select_audio_stream(_active_audio_stream);
     }
 
+    // Set active subtitle stream
+    _active_subtitle_stream = -1;       // no subtitles by default
+
     // Print summary
-    msg::inf("Input:");
+    msg::inf(_("Input:"));
     for (int i = 0; i < video_streams(); i++)
     {
         int o, s;
         get_video_stream(i, o, s);
-        msg::inf("    Video %s: %s", video_stream_name(i).c_str(),
+        msg::inf(4, _("Video %s: %s"), video_stream_name(i).c_str(),
                 _media_objects[o].video_frame_template(s).format_name().c_str());
     }
     if (video_streams() == 0)
     {
-        msg::inf("    No video.");
+        msg::inf(4, _("No video."));
     }
     for (int i = 0; i < audio_streams(); i++)
     {
         int o, s;
         get_audio_stream(i, o, s);
-        msg::inf("    Audio %s: %s", audio_stream_name(i).c_str(), 
+        msg::inf(4, _("Audio %s: %s"), audio_stream_name(i).c_str(),
                 _media_objects[o].audio_blob_template(s).format_name().c_str());
     }
     if (audio_streams() == 0)
     {
-        msg::inf("    No audio.");
+        msg::inf(4, _("No audio."));
     }
-    msg::inf("    Duration: %g seconds", duration() / 1e6f);
+    for (int i = 0; i < subtitle_streams(); i++)
+    {
+        int o, s;
+        get_subtitle_stream(i, o, s);
+        msg::inf(4, _("Subtitle %s: %s"), subtitle_stream_name(i).c_str(),
+                _media_objects[o].subtitle_box_template(s).format_name().c_str());
+    }
+    if (subtitle_streams() == 0)
+    {
+        msg::inf(4, _("No subtitle."));
+    }
+    msg::inf(4, _("Duration: %g seconds"), duration() / 1e6f);
     if (video_streams() > 0)
     {
-        msg::inf("    Stereo layout: %s", video_frame::stereo_layout_to_string(
+        msg::inf(4, _("Stereo layout: %s"), video_frame::stereo_layout_to_string(
                     video_frame_template().stereo_layout, video_frame_template().stereo_layout_swap).c_str());
     }
 }
@@ -328,6 +392,12 @@ const audio_blob &media_input::audio_blob_template() const
     return _audio_blob;
 }
 
+const subtitle_box &media_input::subtitle_box_template() const
+{
+    assert(_active_subtitle_stream >= 0);
+    return _subtitle_box;
+}
+
 bool media_input::stereo_layout_is_supported(video_frame::stereo_layout_t layout, bool) const
 {
     if (video_streams() < 1)
@@ -361,6 +431,10 @@ void media_input::set_stereo_layout(video_frame::stereo_layout_t layout, bool sw
     {
         (void)finish_audio_blob_read();
     }
+    if (_have_active_subtitle_read)
+    {
+        (void)finish_subtitle_box_read();
+    }
     int o, s;
     get_video_stream(_active_video_stream, o, s);
     const video_frame &t = _media_objects[o].video_frame_template(s);
@@ -393,6 +467,10 @@ void media_input::select_video_stream(int video_stream)
     {
         (void)finish_audio_blob_read();
     }
+    if (_have_active_subtitle_read)
+    {
+        (void)finish_subtitle_box_read();
+    }
     assert(video_stream >= 0);
     assert(video_stream < video_streams());
     if (_video_frame.stereo_layout == video_frame::separate)
@@ -431,6 +509,10 @@ void media_input::select_audio_stream(int audio_stream)
     {
         (void)finish_audio_blob_read();
     }
+    if (_have_active_subtitle_read)
+    {
+        (void)finish_subtitle_box_read();
+    }
     assert(audio_stream >= 0);
     assert(audio_stream < audio_streams());
     _active_audio_stream = audio_stream;
@@ -445,6 +527,37 @@ void media_input::select_audio_stream(int audio_stream)
     }
 }
 
+void media_input::select_subtitle_stream(int subtitle_stream)
+{
+    if (_have_active_video_read)
+    {
+        (void)finish_video_frame_read();
+    }
+    if (_have_active_audio_read)
+    {
+        (void)finish_audio_blob_read();
+    }
+    if (_have_active_subtitle_read)
+    {
+        (void)finish_subtitle_box_read();
+    }
+    assert(subtitle_stream >= -1);
+    assert(subtitle_stream < subtitle_streams());
+    _active_subtitle_stream = subtitle_stream;
+    int o = -1, s = -1;
+    if (_active_subtitle_stream >= 0)
+    {
+        get_subtitle_stream(_active_subtitle_stream, o, s);
+    }
+    for (size_t i = 0; i < _media_objects.size(); i++)
+    {
+        for (int j = 0; j < _media_objects[i].subtitle_streams(); j++)
+        {
+            _media_objects[i].subtitle_stream_set_active(j, (i == static_cast<size_t>(o) && j == s));
+        }
+    }
+}
+
 void media_input::start_video_frame_read()
 {
     assert(_active_video_stream >= 0);
@@ -544,6 +657,32 @@ audio_blob media_input::finish_audio_blob_read()
     return _media_objects[o].finish_audio_blob_read(s);
 }
 
+void media_input::start_subtitle_box_read()
+{
+    assert(_active_subtitle_stream >= 0);
+    if (_have_active_subtitle_read)
+    {
+        return;
+    }
+    int o, s;
+    get_subtitle_stream(_active_subtitle_stream, o, s);
+    _media_objects[o].start_subtitle_box_read(s);
+    _have_active_subtitle_read = true;
+}
+
+subtitle_box media_input::finish_subtitle_box_read()
+{
+    assert(_active_subtitle_stream >= 0);
+    int o, s;
+    get_subtitle_stream(_active_subtitle_stream, o, s);
+    if (!_have_active_subtitle_read)
+    {
+        start_subtitle_box_read();
+    }
+    _have_active_subtitle_read = false;
+    return _media_objects[o].finish_subtitle_box_read(s);
+}
+
 int64_t media_input::tell()
 {
     int64_t pos = std::numeric_limits<int64_t>::min();
@@ -571,6 +710,10 @@ void media_input::seek(int64_t pos)
     {
         (void)finish_audio_blob_read();
     }
+    if (_have_active_subtitle_read)
+    {
+        (void)finish_subtitle_box_read();
+    }
     for (size_t i = 0; i < _media_objects.size(); i++)
     {
         _media_objects[i].seek(pos);
@@ -589,6 +732,10 @@ void media_input::close()
         {
             (void)finish_audio_blob_read();
         }
+        if (_have_active_subtitle_read)
+        {
+            (void)finish_subtitle_box_read();
+        }
         for (size_t i = 0; i < _media_objects.size(); i++)
         {
             _media_objects[i].close();
@@ -603,10 +750,13 @@ void media_input::close()
     _tag_values.clear();
     _video_stream_names.clear();
     _audio_stream_names.clear();
+    _subtitle_stream_names.clear();
     _active_video_stream = -1;
     _active_audio_stream = -1;
+    _active_subtitle_stream = -1;
     _initial_skip = 0;
     _duration = -1;
     _video_frame = video_frame();
     _audio_blob = audio_blob();
+    _subtitle_box = subtitle_box();
 }
