@@ -4,6 +4,7 @@
  * Copyright (C) 2010-2011
  * Martin Lambers <marlam@marlam.de>
  * Frédéric Devernay <frederic.devernay@inrialpes.fr>
+ * Joe <cuchac@email.cz>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -41,6 +42,9 @@ extern "C"
 #  include <windows.h>
 #endif
 
+#include "gettext.h"
+#define _(string) gettext(string)
+
 #include "dbg.h"
 #include "blob.h"
 #include "exc.h"
@@ -113,6 +117,27 @@ public:
     }
 };
 
+// The subtitle decode thread.
+// This thread reads packets from its packet queue and decodes them to subtitle boxes.
+class subtitle_decode_thread : public thread
+{
+private:
+    std::string _url;
+    struct ffmpeg_stuff *_ffmpeg;
+    int _subtitle_stream;
+    subtitle_box _box;
+
+    int64_t handle_timestamp(int64_t timestamp);
+
+public:
+    subtitle_decode_thread(const std::string &url, struct ffmpeg_stuff *ffmpeg, int subtitle_stream);
+    void run();
+    const subtitle_box &box()
+    {
+        return _box;
+    }
+};
+
 
 // Hide the FFmpeg stuff so that their messy header files cannot cause problems
 // in other source files.
@@ -153,6 +178,16 @@ struct ffmpeg_stuff
     std::vector<blob> audio_blobs;
     std::vector<std::vector<unsigned char> > audio_buffers;
     std::vector<int64_t> audio_last_timestamps;
+
+    std::vector<int> subtitle_streams;
+    std::vector<AVCodecContext *> subtitle_codec_ctxs;
+    std::vector<subtitle_box> subtitle_box_templates;
+    std::vector<AVCodec *> subtitle_codecs;
+    std::vector<std::deque<AVPacket> > subtitle_packet_queues;
+    std::vector<mutex> subtitle_packet_queue_mutexes;
+    std::vector<subtitle_decode_thread> subtitle_decode_threads;
+    std::vector<std::deque<subtitle_box> > subtitle_box_buffers;
+    std::vector<int64_t> subtitle_last_timestamps;
 };
 
 // Use one decoding thread per processor for video decoding.
@@ -259,6 +294,25 @@ static int64_t timestamp_helper(int64_t &last_timestamp, int64_t timestamp)
     return timestamp;
 }
 
+// Get a stream duration
+static int64_t stream_duration(AVStream *stream, AVFormatContext *format)
+{
+    // Try to get duration from the stream first. If that fails, fall back to
+    // the value provided by the container.
+    int64_t duration = stream->duration;
+    if (duration > 0)
+    {
+        AVRational time_base = stream->time_base;
+        return duration * 1000000 * time_base.num / time_base.den;
+    }
+    else
+    {
+        duration = format->duration;
+        return duration * 1000000 / AV_TIME_BASE;
+    }
+}
+
+
 media_object::media_object() : _ffmpeg(NULL)
 {
     av_register_all();
@@ -517,14 +571,18 @@ void media_object::set_audio_blob_template(int index)
     AVCodecContext *audio_codec_ctx = _ffmpeg->audio_codec_ctxs[index];
     audio_blob &audio_blob_template = _ffmpeg->audio_blob_templates[index];
 
+    AVMetadataTag *tag = av_metadata_get(audio_stream->metadata, "language", NULL, AV_METADATA_IGNORE_SUFFIX);
+    if (tag)
+    {
+        audio_blob_template.language = tag->value;
+    }
     if (audio_codec_ctx->channels < 1
             || audio_codec_ctx->channels > 8
             || audio_codec_ctx->channels == 3
             || audio_codec_ctx->channels == 5)
     {
-        throw exc(_url + " audio stream " + str::from(audio_stream)
-                + ": Cannot handle audio with "
-                + str::from(audio_codec_ctx->channels) + " channels.");
+        throw exc(str::asprintf(_("%s audio stream %d: Cannot handle audio with %d channels."),
+                    _url.c_str(), index + 1, audio_codec_ctx->channels));
     }
     audio_blob_template.channels = audio_codec_ctx->channels;
     audio_blob_template.rate = audio_codec_ctx->sample_rate;
@@ -546,11 +604,21 @@ void media_object::set_audio_blob_template(int index)
     }
     else
     {
-        throw exc(_url + " audio stream " + str::from(audio_stream)
-                + ": Cannot handle audio with sample format "
-                + str::from(static_cast<int>(audio_codec_ctx->sample_fmt)) + ".");
-        // XXX: Once widely available, use av_get_sample_fmt_name(audio_codec_ctx->sample_fmt)
-        // in this error message.
+        throw exc(str::asprintf(_("%s audio stream %d: Cannot handle audio with sample format %s."),
+                    _url.c_str(), index + 1, av_get_sample_fmt_name(audio_codec_ctx->sample_fmt)));
+    }
+}
+
+void media_object::set_subtitle_box_template(int index)
+{
+    AVStream *subtitle_stream = _ffmpeg->format_ctx->streams[_ffmpeg->subtitle_streams[index]];
+    //AVCodecContext *subtitle_codec_ctx = _ffmpeg->subtitle_codec_ctxs[index];
+    subtitle_box &subtitle_box_template = _ffmpeg->subtitle_box_templates[index];
+
+    AVMetadataTag *tag = av_metadata_get(subtitle_stream->metadata, "language", NULL, AV_METADATA_IGNORE_SUFFIX);
+    if (tag)
+    {
+        subtitle_box_template.language = tag->value;
     }
 }
 
@@ -565,13 +633,15 @@ void media_object::open(const std::string &url)
 
     if ((e = av_open_input_file(&_ffmpeg->format_ctx, _url.c_str(), NULL, 0, NULL)) != 0)
     {
-        throw exc(_url + ": " + my_av_strerror(e));
+        throw exc(str::asprintf(_("%s: %s"),
+                    _url.c_str(), my_av_strerror(e).c_str()));
     }
     if ((e = av_find_stream_info(_ffmpeg->format_ctx)) < 0)
     {
-        throw exc(_url + ": Cannot read stream info: " + my_av_strerror(e));
+        throw exc(str::asprintf(_("%s: Cannot read stream info: %s"),
+                    _url.c_str(), my_av_strerror(e).c_str()));
     }
-    dump_format(_ffmpeg->format_ctx, 0, _url.c_str(), 0);
+    av_dump_format(_ffmpeg->format_ctx, 0, _url.c_str(), 0);
 
     /* Metadata */
     AVMetadataTag *tag = NULL;
@@ -596,22 +666,21 @@ void media_object::open(const std::string &url)
             _ffmpeg->video_codec_ctxs.push_back(_ffmpeg->format_ctx->streams[i]->codec);
             if (_ffmpeg->video_codec_ctxs[j]->width < 1 || _ffmpeg->video_codec_ctxs[j]->height < 1)
             {
-                throw exc(_url + " stream " + str::from(i) + ": Invalid frame size.");
-            }
-            _ffmpeg->video_codec_ctxs[j]->thread_count = video_decoding_threads();
-            if (avcodec_thread_init(_ffmpeg->video_codec_ctxs[j], _ffmpeg->video_codec_ctxs[j]->thread_count) != 0)
-            {
-                _ffmpeg->video_codec_ctxs[j]->thread_count = 1;
+                throw exc(str::asprintf(_("%s video stream %d: Invalid frame size."),
+                            _url.c_str(), j + 1));
             }
             _ffmpeg->video_codecs.push_back(avcodec_find_decoder(_ffmpeg->video_codec_ctxs[j]->codec_id));
             if (!_ffmpeg->video_codecs[j])
             {
-                throw exc(_url + " stream " + str::from(i) + ": Unsupported video codec.");
+                throw exc(str::asprintf(_("%s video stream %d: Unsupported video codec."),
+                            _url.c_str(), j + 1));
             }
+            _ffmpeg->video_codec_ctxs[j]->thread_count = video_decoding_threads();
             if ((e = avcodec_open(_ffmpeg->video_codec_ctxs[j], _ffmpeg->video_codecs[j])) < 0)
             {
                 _ffmpeg->video_codecs[j] = NULL;
-                throw exc(_url + " stream " + str::from(i) + ": Cannot open video codec: " + my_av_strerror(e));
+                throw exc(str::asprintf(_("%s video stream %d: Cannot open video codec: %s"),
+                            _url.c_str(), j + 1, my_av_strerror(e).c_str()));
             }
             // Determine frame template.
             _ffmpeg->video_frame_templates.push_back(video_frame());
@@ -638,13 +707,15 @@ void media_object::open(const std::string &url)
                 }
                 avpicture_fill(reinterpret_cast<AVPicture *>(_ffmpeg->video_out_frames[j]), _ffmpeg->video_buffers[j],
                         PIX_FMT_BGRA, _ffmpeg->video_codec_ctxs[j]->width, _ffmpeg->video_codec_ctxs[j]->height);
-                _ffmpeg->video_img_conv_ctxs.push_back(sws_getContext(
+                // Call sws_getCachedContext(NULL, ...) instead of sws_getContext(...) just to avoid a deprecation warning.
+                _ffmpeg->video_img_conv_ctxs.push_back(sws_getCachedContext(NULL,
                             _ffmpeg->video_codec_ctxs[j]->width, _ffmpeg->video_codec_ctxs[j]->height, _ffmpeg->video_codec_ctxs[j]->pix_fmt,
                             _ffmpeg->video_codec_ctxs[j]->width, _ffmpeg->video_codec_ctxs[j]->height, PIX_FMT_BGRA,
                             SWS_POINT, NULL, NULL, NULL));
                 if (!_ffmpeg->video_img_conv_ctxs[j])
                 {
-                    throw exc(_url + " stream " + str::from(i) + ": Cannot initialize conversion context.");
+                    throw exc(str::asprintf(_("%s video stream %d: Cannot initialize conversion context."),
+                                _url.c_str(), j + 1));
                 }
             }
             else
@@ -664,12 +735,14 @@ void media_object::open(const std::string &url)
             _ffmpeg->audio_codecs.push_back(avcodec_find_decoder(_ffmpeg->audio_codec_ctxs[j]->codec_id));
             if (!_ffmpeg->audio_codecs[j])
             {
-                throw exc(_url + " stream " + str::from(i) + ": Unsupported audio codec.");
+                throw exc(str::asprintf(_("%s audio stream %d: Unsupported audio codec."),
+                            _url.c_str(), j + 1));
             }
             if ((e = avcodec_open(_ffmpeg->audio_codec_ctxs[j], _ffmpeg->audio_codecs[j])) < 0)
             {
                 _ffmpeg->audio_codecs[j] = NULL;
-                throw exc(_url + " stream " + str::from(i) + ": Cannot open audio codec: " + my_av_strerror(e));
+                throw exc(str::asprintf(_("%s audio stream %d: Cannot open audio codec: %s"),
+                            _url.c_str(), j + 1, my_av_strerror(e).c_str()));
             }
             _ffmpeg->audio_blob_templates.push_back(audio_blob());
             set_audio_blob_template(j);
@@ -685,36 +758,77 @@ void media_object::open(const std::string &url)
             _ffmpeg->audio_buffers.push_back(std::vector<unsigned char>());
             _ffmpeg->audio_last_timestamps.push_back(std::numeric_limits<int64_t>::min());
         }
+        else if (_ffmpeg->format_ctx->streams[i]->codec->codec_type == CODEC_TYPE_SUBTITLE)
+        {
+            _ffmpeg->subtitle_streams.push_back(i);
+            int j = _ffmpeg->subtitle_streams.size() - 1;
+            msg::dbg(_url + " stream " + str::from(i) + " is subtitle stream " + str::from(j) + ".");
+            _ffmpeg->subtitle_codec_ctxs.push_back(_ffmpeg->format_ctx->streams[i]->codec);
+            if (_ffmpeg->subtitle_codec_ctxs[j]->codec_id == CODEC_ID_TEXT)
+            {
+                // CODEC_ID_TEXT does not have any decoder; it is just UTF-8 text in the packet data.
+                _ffmpeg->subtitle_codecs.push_back(NULL);
+            }
+            else
+            {
+                _ffmpeg->subtitle_codecs.push_back(avcodec_find_decoder(_ffmpeg->subtitle_codec_ctxs[j]->codec_id));
+                if (!_ffmpeg->subtitle_codecs[j])
+                {
+                    throw exc(str::asprintf(_("%s subtitle stream %d: Unsupported subtitle codec."),
+                                _url.c_str(), j + 1));
+                }
+                if ((e = avcodec_open(_ffmpeg->subtitle_codec_ctxs[j], _ffmpeg->subtitle_codecs[j])) < 0)
+                {
+                    _ffmpeg->subtitle_codecs[j] = NULL;
+                    throw exc(str::asprintf(_("%s subtitle stream %d: Cannot open subtitle codec: %s"),
+                                _url.c_str(), j + 1, my_av_strerror(e).c_str()));
+                }
+            }
+            _ffmpeg->subtitle_box_templates.push_back(subtitle_box());
+            set_subtitle_box_template(j);
+            _ffmpeg->subtitle_decode_threads.push_back(subtitle_decode_thread(_url, _ffmpeg, j));
+            _ffmpeg->subtitle_box_buffers.push_back(std::deque<subtitle_box>());
+            _ffmpeg->subtitle_last_timestamps.push_back(std::numeric_limits<int64_t>::min());
+        }
         else
         {
-            msg::dbg(_url + " stream " + str::from(i) + " contains neither video nor audio.");
+            msg::dbg(_url + " stream " + str::from(i) + " contains neither video nor audio nor subtitles.");
         }
     }
     _ffmpeg->video_packet_queues.resize(video_streams());
     _ffmpeg->audio_packet_queues.resize(audio_streams());
+    _ffmpeg->subtitle_packet_queues.resize(subtitle_streams());
     _ffmpeg->video_packet_queue_mutexes.resize(video_streams());
     _ffmpeg->audio_packet_queue_mutexes.resize(audio_streams());
+    _ffmpeg->subtitle_packet_queue_mutexes.resize(subtitle_streams());
 
     msg::inf(_url + ":");
     for (int i = 0; i < video_streams(); i++)
     {
-        msg::inf("    Video stream %d: %s / %s, %g seconds", i,
+        msg::inf(4, _("Video stream %d: %s / %s, %g seconds"), i,
                 video_frame_template(i).format_info().c_str(),
                 video_frame_template(i).format_name().c_str(),
                 video_duration(i) / 1e6f);
-        msg::inf("        Using up to %d threads for decoding.",
+        msg::inf(8, _("Using up to %d threads for decoding."),
                 _ffmpeg->video_codec_ctxs.at(i)->thread_count);
     }
     for (int i = 0; i < audio_streams(); i++)
     {
-        msg::inf("    Audio stream %d: %s / %s, %g seconds", i,
+        msg::inf(4, _("Audio stream %d: %s / %s, %g seconds"), i,
                 audio_blob_template(i).format_info().c_str(),
                 audio_blob_template(i).format_name().c_str(),
                 audio_duration(i) / 1e6f);
     }
-    if (video_streams() == 0 && audio_streams() == 0)
+    for (int i = 0; i < subtitle_streams(); i++)
     {
-        msg::inf("    No usable streams.");
+        msg::inf(4, _("Subtitle stream %d: %s / %s, %g seconds"), i,
+                subtitle_box_template(i).format_info().c_str(),
+                subtitle_box_template(i).format_name().c_str(),
+                subtitle_duration(i) / 1e6f);
+    }
+    if (video_streams() == 0 && audio_streams() == 0 && subtitle_streams() == 0)
+    {
+        msg::inf(4, _("No usable streams."));
     }
 }
 
@@ -753,14 +867,19 @@ const std::string &media_object::tag_value(const std::string &tag_name) const
     return empty;
 }
 
+int media_object::video_streams() const
+{
+    return _ffmpeg->video_streams.size();
+}
+
 int media_object::audio_streams() const
 {
     return _ffmpeg->audio_streams.size();
 }
 
-int media_object::video_streams() const
+int media_object::subtitle_streams() const
 {
-    return _ffmpeg->video_streams.size();
+    return _ffmpeg->subtitle_streams.size();
 }
 
 void media_object::video_stream_set_active(int index, bool active)
@@ -776,6 +895,10 @@ void media_object::video_stream_set_active(int index, bool active)
     {
         _ffmpeg->audio_decode_threads[i].finish();
     }
+    for (size_t i = 0; i < _ffmpeg->subtitle_streams.size(); i++)
+    {
+        _ffmpeg->subtitle_decode_threads[i].finish();
+    }
     // Stop reading packets
     _ffmpeg->reader->finish();
     // Set status
@@ -798,6 +921,10 @@ void media_object::audio_stream_set_active(int index, bool active)
     {
         _ffmpeg->audio_decode_threads[i].finish();
     }
+    for (size_t i = 0; i < _ffmpeg->subtitle_streams.size(); i++)
+    {
+        _ffmpeg->subtitle_decode_threads[i].finish();
+    }
     // Stop reading packets
     _ffmpeg->reader->finish();
     // Set status
@@ -816,6 +943,32 @@ void media_object::audio_stream_set_active(int index, bool active)
     _ffmpeg->reader->start();
 }
 
+void media_object::subtitle_stream_set_active(int index, bool active)
+{
+    assert(index >= 0);
+    assert(index < subtitle_streams());
+    // Stop decoder threads
+    for (size_t i = 0; i < _ffmpeg->video_streams.size(); i++)
+    {
+        _ffmpeg->video_decode_threads[i].finish();
+    }
+    for (size_t i = 0; i < _ffmpeg->audio_streams.size(); i++)
+    {
+        _ffmpeg->audio_decode_threads[i].finish();
+    }
+    for (size_t i = 0; i < _ffmpeg->subtitle_streams.size(); i++)
+    {
+        _ffmpeg->subtitle_decode_threads[i].finish();
+    }
+    // Stop reading packets
+    _ffmpeg->reader->finish();
+    // Set status
+    _ffmpeg->format_ctx->streams[_ffmpeg->subtitle_streams.at(index)]->discard =
+        (active ? AVDISCARD_DEFAULT : AVDISCARD_ALL);
+    // Restart reader
+    _ffmpeg->reader->start();
+}
+
 const video_frame &media_object::video_frame_template(int video_stream) const
 {
     assert(video_stream >= 0);
@@ -841,19 +994,9 @@ int64_t media_object::video_duration(int index) const
 {
     assert(index >= 0);
     assert(index < video_streams());
-    // Try to get duration from the Codec first. If that fails, fall back to
-    // the value provided by the container.
-    int64_t duration = _ffmpeg->format_ctx->streams[_ffmpeg->video_streams.at(index)]->duration;
-    if (duration > 0)
-    {
-        AVRational time_base = _ffmpeg->format_ctx->streams[_ffmpeg->video_streams.at(index)]->time_base;
-        return duration * 1000000 * time_base.num / time_base.den;
-    }
-    else
-    {
-        duration = _ffmpeg->format_ctx->duration;
-        return duration * 1000000 / AV_TIME_BASE;
-    }
+    return stream_duration(
+            _ffmpeg->format_ctx->streams[_ffmpeg->video_streams.at(index)],
+            _ffmpeg->format_ctx);
 }
 
 const audio_blob &media_object::audio_blob_template(int audio_stream) const
@@ -867,19 +1010,25 @@ int64_t media_object::audio_duration(int index) const
 {
     assert(index >= 0);
     assert(index < audio_streams());
-    // Try to get duration from the Codec first. If that fails, fall back to
-    // the value provided by the container.
-    int64_t duration = _ffmpeg->format_ctx->streams[_ffmpeg->audio_streams.at(index)]->duration;
-    if (duration > 0)
-    {
-        AVRational time_base = _ffmpeg->format_ctx->streams[_ffmpeg->audio_streams.at(index)]->time_base;
-        return duration * 1000000 * time_base.num / time_base.den;
-    }
-    else
-    {
-        duration = _ffmpeg->format_ctx->duration;
-        return duration * 1000000 / AV_TIME_BASE;
-    }
+    return stream_duration(
+            _ffmpeg->format_ctx->streams[_ffmpeg->audio_streams.at(index)],
+            _ffmpeg->format_ctx);
+}
+
+const subtitle_box &media_object::subtitle_box_template(int subtitle_stream) const
+{
+    assert(subtitle_stream >= 0);
+    assert(subtitle_stream < subtitle_streams());
+    return _ffmpeg->subtitle_box_templates.at(subtitle_stream);
+}
+
+int64_t media_object::subtitle_duration(int index) const
+{
+    assert(index >= 0);
+    assert(index < subtitle_streams());
+    return stream_duration(
+            _ffmpeg->format_ctx->streams[_ffmpeg->subtitle_streams.at(index)],
+            _ffmpeg->format_ctx);
 }
 
 read_thread::read_thread(const std::string &url, struct ffmpeg_stuff *ffmpeg) :
@@ -894,6 +1043,7 @@ void read_thread::run()
         // We need another packet if the number of queued packets for an active stream is below a threshold.
         const size_t video_stream_low_threshold = 2;    // Often, 1 packet results in one video frame
         const size_t audio_stream_low_threshold = 5;    // Often, 3-4 packets are needed for one buffer fill
+        const size_t subtitle_stream_low_threshold = 1; // Just a guess
         bool need_another_packet = false;
         for (size_t i = 0; !need_another_packet && i < _ffmpeg->video_streams.size(); i++)
         {
@@ -913,6 +1063,15 @@ void read_thread::run()
                 _ffmpeg->audio_packet_queue_mutexes[i].unlock();
             }
         }
+        for (size_t i = 0; !need_another_packet && i < _ffmpeg->subtitle_streams.size(); i++)
+        {
+            if (_ffmpeg->format_ctx->streams[_ffmpeg->subtitle_streams[i]]->discard == AVDISCARD_DEFAULT)
+            {
+                _ffmpeg->subtitle_packet_queue_mutexes[i].lock();
+                need_another_packet = _ffmpeg->subtitle_packet_queues[i].size() < subtitle_stream_low_threshold;
+                _ffmpeg->subtitle_packet_queue_mutexes[i].unlock();
+            }
+        }
         if (!need_another_packet)
         {
             msg::dbg(_url + ": No need to read more packets.");
@@ -932,7 +1091,7 @@ void read_thread::run()
             }
             else
             {
-                throw exc(_url + ": " + my_av_strerror(e));
+                throw exc(str::asprintf(_("%s: %s"), _url.c_str(), my_av_strerror(e).c_str()));
             }
         }
         // Put the packet in the right queue.
@@ -948,7 +1107,7 @@ void read_thread::run()
                 //    interdependencies. We would mess up decoding.
                 if (av_dup_packet(&packet) < 0)
                 {
-                    throw exc(_url + ": Cannot duplicate packet.");
+                    throw exc(str::asprintf(_("%s: Cannot duplicate packet."), _url.c_str()));
                 }
                 _ffmpeg->video_packet_queue_mutexes[i].lock();
                 _ffmpeg->video_packet_queues[i].push_back(packet);
@@ -978,7 +1137,7 @@ void read_thread::run()
                     if (av_dup_packet(&packet) < 0)
                     {
                         _ffmpeg->audio_packet_queue_mutexes[i].unlock();
-                        throw exc(_url + ": Cannot duplicate packet.");
+                        throw exc(str::asprintf(_("%s: Cannot duplicate packet."), _url.c_str()));
                     }
                     _ffmpeg->audio_packet_queues[i].push_back(packet);
                     packet_queued = true;
@@ -989,6 +1148,36 @@ void read_thread::run()
                 _ffmpeg->audio_packet_queue_mutexes[i].unlock();
             }
         }
+        for (size_t i = 0; i < _ffmpeg->subtitle_streams.size() && !packet_queued; i++)
+        {
+            if (packet.stream_index == _ffmpeg->subtitle_streams[i])
+            {
+                _ffmpeg->subtitle_packet_queue_mutexes[i].lock();
+                if (_ffmpeg->subtitle_packet_queues[i].empty()
+                        && _ffmpeg->subtitle_last_timestamps[i] == std::numeric_limits<int64_t>::min()
+                        && packet.dts == static_cast<int64_t>(AV_NOPTS_VALUE))
+                {
+                    // We have no packet in the queue and no last timestamp, probably
+                    // because we just seeked. We want a packet with a timestamp.
+                    msg::dbg(_url + ": subtitle stream " + str::from(i)
+                            + ": dropping packet because it has no timestamp");
+                }
+                else
+                {
+                    if (av_dup_packet(&packet) < 0)
+                    {
+                        _ffmpeg->subtitle_packet_queue_mutexes[i].unlock();
+                        throw exc(str::asprintf(_("%s: Cannot duplicate packet."), _url.c_str()));
+                    }
+                    _ffmpeg->subtitle_packet_queues[i].push_back(packet);
+                    packet_queued = true;
+                    msg::dbg(_url + ": "
+                            + str::from(_ffmpeg->subtitle_packet_queues[i].size())
+                            + " packets queued in subtitle stream " + str::from(i) + ".");
+                }
+                _ffmpeg->subtitle_packet_queue_mutexes[i].unlock();
+            }
+        }
         if (!packet_queued)
         {
             av_free_packet(&packet);
@@ -1084,13 +1273,13 @@ void video_decode_thread::run()
     }
     else if (_ffmpeg->video_last_timestamps[_video_stream] != std::numeric_limits<int64_t>::min())
     {
-        msg::wrn(_url + ": video stream " + str::from(_video_stream)
+        msg::dbg(_url + ": video stream " + str::from(_video_stream)
                 + ": no timestamp available, using a questionable guess");
         _frame.presentation_time = _ffmpeg->video_last_timestamps[_video_stream];
     }
     else
     {
-        msg::wrn(_url + ": video stream " + str::from(_video_stream)
+        msg::dbg(_url + ": video stream " + str::from(_video_stream)
                 + ": no timestamp available, using a bad guess");
         _frame.presentation_time = _ffmpeg->pos;
     }
@@ -1200,7 +1389,7 @@ void audio_decode_thread::run()
                 _ffmpeg->audio_buffers[_audio_stream].resize(old_size + tmpbuf_size);
                 memcpy(&(_ffmpeg->audio_buffers[_audio_stream][old_size]), _ffmpeg->audio_tmpbufs[_audio_stream], tmpbuf_size);
             }
-            
+
             av_free_packet(&packet);
         }
     }
@@ -1210,7 +1399,7 @@ void audio_decode_thread::run()
     }
     if (timestamp == std::numeric_limits<int64_t>::min())
     {
-        msg::wrn(_url + ": audio stream " + str::from(_audio_stream)
+        msg::dbg(_url + ": audio stream " + str::from(_audio_stream)
                 + ": no timestamp available, using a bad guess");
         timestamp = _ffmpeg->pos;
     }
@@ -1237,6 +1426,194 @@ audio_blob media_object::finish_audio_blob_read(int audio_stream)
     return _ffmpeg->audio_decode_threads[audio_stream].blob();
 }
 
+subtitle_decode_thread::subtitle_decode_thread(const std::string &url, struct ffmpeg_stuff *ffmpeg, int subtitle_stream) :
+    _url(url), _ffmpeg(ffmpeg), _subtitle_stream(subtitle_stream), _box()
+{
+}
+
+int64_t subtitle_decode_thread::handle_timestamp(int64_t timestamp)
+{
+    int64_t ts = timestamp_helper(_ffmpeg->subtitle_last_timestamps[_subtitle_stream], timestamp);
+    _ffmpeg->pos = ts;
+    return ts;
+}
+
+void subtitle_decode_thread::run()
+{
+    if (_ffmpeg->subtitle_box_buffers[_subtitle_stream].empty())
+    {
+        // Read more subtitle data
+        AVPacket packet, tmppacket;
+        bool empty;
+        do
+        {
+            _ffmpeg->subtitle_packet_queue_mutexes[_subtitle_stream].lock();
+            empty = _ffmpeg->subtitle_packet_queues[_subtitle_stream].empty();
+            _ffmpeg->subtitle_packet_queue_mutexes[_subtitle_stream].unlock();
+            if (empty)
+            {
+                if (_ffmpeg->reader->eof())
+                {
+                    _box = subtitle_box();
+                    return;
+                }
+                msg::dbg(_url + ": subtitle stream " + str::from(_subtitle_stream) + ": need to wait for packets...");
+                _ffmpeg->reader->start();
+                _ffmpeg->reader->finish();
+            }
+        }
+        while (empty);
+        _ffmpeg->subtitle_packet_queue_mutexes[_subtitle_stream].lock();
+        packet = _ffmpeg->subtitle_packet_queues[_subtitle_stream].front();
+        _ffmpeg->subtitle_packet_queues[_subtitle_stream].pop_front();
+        _ffmpeg->subtitle_packet_queue_mutexes[_subtitle_stream].unlock();
+        _ffmpeg->reader->start();   // Refill the packet queue
+
+        // Decode subtitle data
+        int64_t timestamp = packet.pts * 1000000
+            * _ffmpeg->format_ctx->streams[_ffmpeg->subtitle_streams[_subtitle_stream]]->time_base.num
+            / _ffmpeg->format_ctx->streams[_ffmpeg->subtitle_streams[_subtitle_stream]]->time_base.den;
+        AVSubtitle subtitle;
+        int got_subtitle;
+        tmppacket = packet;
+
+        // CODEC_ID_TEXT does not have any decoder; it is just UTF-8 text in the packet data.
+        if (_ffmpeg->subtitle_codec_ctxs[_subtitle_stream]->codec_id == CODEC_ID_TEXT)
+        {
+            int64_t duration = packet.convergence_duration * 1000000
+                * _ffmpeg->format_ctx->streams[_ffmpeg->subtitle_streams[_subtitle_stream]]->time_base.num
+                / _ffmpeg->format_ctx->streams[_ffmpeg->subtitle_streams[_subtitle_stream]]->time_base.den;
+
+            // Put it in the subtitle buffer
+            subtitle_box box = _ffmpeg->subtitle_box_templates[_subtitle_stream];
+            box.presentation_start_time = timestamp;
+            box.presentation_stop_time = timestamp + duration;
+
+            box.format = subtitle_box::text;
+            box.str = reinterpret_cast<const char *>(packet.data);
+
+            _ffmpeg->subtitle_box_buffers[_subtitle_stream].push_back(box);
+
+            tmppacket.size = 0;
+        }
+
+        while (tmppacket.size > 0)
+        {
+            int len = avcodec_decode_subtitle2(_ffmpeg->subtitle_codec_ctxs[_subtitle_stream],
+                    &subtitle, &got_subtitle, &tmppacket);
+            if (len < 0)
+            {
+                tmppacket.size = 0;
+                break;
+            }
+            tmppacket.data += len;
+            tmppacket.size -= len;
+            if (!got_subtitle)
+            {
+                continue;
+            }
+            // Put it in the subtitle buffer
+            subtitle_box box = _ffmpeg->subtitle_box_templates[_subtitle_stream];
+            box.presentation_start_time = timestamp + subtitle.start_display_time * 1000;
+            box.presentation_stop_time = box.presentation_start_time + subtitle.end_display_time * 1000;
+            if (subtitle.num_rects > 0 && subtitle.rects[0]->type == SUBTITLE_ASS
+                    && subtitle.start_display_time == subtitle.end_display_time)
+            {
+                // XXX As of 20110409, FFmpeg does not properly set the duration of ASS subtitles.
+                // This is a temporary workaround that can be removed once a fixed version of
+                // FFmpeg can be assumed. See http://thread.gmane.org/gmane.comp.video.ffmpeg.devel/130619
+                int start_hour, start_min, start_sec, start_hsec;
+                int end_hour, end_min, end_sec, end_hsec;
+                if (sscanf(subtitle.rects[0]->ass, "%*[^,],%d:%d:%d%*c%d,%d:%d:%d%*c%d",
+                            &start_hour, &start_min, &start_sec, &start_hsec,
+                            &end_hour, &end_min, &end_sec, &end_hsec) == 8)
+                {
+                    start_min += 60 * start_hour;
+                    start_sec += 60 * start_min;
+                    start_hsec += start_sec * 100;
+                    end_min += 60 * end_hour;
+                    end_sec += 60 * end_min;
+                    end_hsec += end_sec * 100;
+                    int64_t duration_ms = (end_hsec - start_hsec) * 10;
+                    box.presentation_stop_time = box.presentation_start_time + duration_ms * 1000;
+                }
+            }
+            for (unsigned int i = 0; i < subtitle.num_rects; i++)
+            {
+                AVSubtitleRect *rect = subtitle.rects[i];
+                switch (rect->type)
+                {
+                case SUBTITLE_BITMAP:
+                    box.format = subtitle_box::image;
+                    box.images.push_back(subtitle_box::image_t());
+                    box.images.back().w = rect->w;
+                    box.images.back().h = rect->h;
+                    box.images.back().x = rect->x;
+                    box.images.back().y = rect->y;
+                    box.images.back().palette.resize(4 * rect->nb_colors);
+                    std::memcpy(&(box.images.back().palette[0]), rect->pict.data[1],
+                            box.images.back().palette.size());
+                    box.images.back().linesize = rect->pict.linesize[0];
+                    box.images.back().data.resize(box.images.back().h * box.images.back().linesize);
+                    std::memcpy(&(box.images.back().data[0]), rect->pict.data[0],
+                            box.images.back().data.size() * sizeof(uint8_t));
+                    break;
+                case SUBTITLE_TEXT:
+                    box.format = subtitle_box::text;
+                    if (!box.str.empty())
+                    {
+                        box.str += '\n';
+                    }
+                    box.str += rect->text;
+                    break;
+                case SUBTITLE_ASS:
+                    box.format = subtitle_box::ass;
+                    box.style = std::string(reinterpret_cast<const char *>(
+                                _ffmpeg->subtitle_codec_ctxs[_subtitle_stream]->subtitle_header),
+                            _ffmpeg->subtitle_codec_ctxs[_subtitle_stream]->subtitle_header_size);
+                    if (!box.str.empty())
+                    {
+                        box.str += '\n';
+                    }
+                    box.str += rect->ass;
+                    break;
+                case SUBTITLE_NONE:
+                    // Should never happen, but make sure we have a valid subtitle box anyway.
+                    box.format = subtitle_box::text;
+                    box.str = ' ';
+                    break;
+                }
+            }
+            _ffmpeg->subtitle_box_buffers[_subtitle_stream].push_back(box);
+            avsubtitle_free(&subtitle);
+        }
+
+        av_free_packet(&packet);
+    }
+    if (_ffmpeg->subtitle_box_buffers[_subtitle_stream].empty())
+    {
+        _box = subtitle_box();
+        return;
+    }
+    _box = _ffmpeg->subtitle_box_buffers[_subtitle_stream].front();
+    _ffmpeg->subtitle_box_buffers[_subtitle_stream].pop_front();
+}
+
+void media_object::start_subtitle_box_read(int subtitle_stream)
+{
+    assert(subtitle_stream >= 0);
+    assert(subtitle_stream < subtitle_streams());
+    _ffmpeg->subtitle_decode_threads[subtitle_stream].start();
+}
+
+subtitle_box media_object::finish_subtitle_box_read(int subtitle_stream)
+{
+    assert(subtitle_stream >= 0);
+    assert(subtitle_stream < subtitle_streams());
+    _ffmpeg->subtitle_decode_threads[subtitle_stream].finish();
+    return _ffmpeg->subtitle_decode_threads[subtitle_stream].box();
+}
+
 int64_t media_object::tell()
 {
     return _ffmpeg->pos;
@@ -1255,6 +1632,10 @@ void media_object::seek(int64_t dest_pos)
     {
         _ffmpeg->audio_decode_threads[i].finish();
     }
+    for (size_t i = 0; i < _ffmpeg->subtitle_streams.size(); i++)
+    {
+        _ffmpeg->subtitle_decode_threads[i].finish();
+    }
     // Stop reading packets
     _ffmpeg->reader->finish();
     // Seek
@@ -1263,7 +1644,7 @@ void media_object::seek(int64_t dest_pos)
             dest_pos < _ffmpeg->pos ?  AVSEEK_FLAG_BACKWARD : 0);
     if (e < 0)
     {
-        msg::err(_url + ": Seeking failed.");
+        msg::err(_("%s: Seeking failed."), _url.c_str());
     }
     // Throw away all queued packets
     for (size_t i = 0; i < _ffmpeg->video_streams.size(); i++)
@@ -1285,6 +1666,20 @@ void media_object::seek(int64_t dest_pos)
         }
         _ffmpeg->audio_packet_queues[i].clear();
     }
+    for (size_t i = 0; i < _ffmpeg->subtitle_streams.size(); i++)
+    {
+        if (_ffmpeg->format_ctx->streams[_ffmpeg->subtitle_streams[i]]->codec->codec_id != CODEC_ID_TEXT)
+        {
+            // CODEC_ID_TEXT has no decoder, so we cannot flush its buffers
+            avcodec_flush_buffers(_ffmpeg->format_ctx->streams[_ffmpeg->subtitle_streams[i]]->codec);
+        }
+        _ffmpeg->subtitle_box_buffers[i].clear();
+        for (size_t j = 0; j < _ffmpeg->subtitle_packet_queues[i].size(); j++)
+        {
+            av_free_packet(&_ffmpeg->subtitle_packet_queues[i][j]);
+        }
+        _ffmpeg->subtitle_packet_queues[i].clear();
+    }
     // The next read request must update the position
     for (size_t i = 0; i < _ffmpeg->video_streams.size(); i++)
     {
@@ -1294,6 +1689,10 @@ void media_object::seek(int64_t dest_pos)
     {
         _ffmpeg->audio_last_timestamps[i] = std::numeric_limits<int64_t>::min();
     }
+    for (size_t i = 0; i < _ffmpeg->subtitle_streams.size(); i++)
+    {
+        _ffmpeg->subtitle_last_timestamps[i] = std::numeric_limits<int64_t>::min();
+    }
     _ffmpeg->pos = std::numeric_limits<int64_t>::min();
     // Restart packet reading
     _ffmpeg->reader->reset();
@@ -1305,14 +1704,18 @@ void media_object::close()
     try
     {
         // Stop decoder threads
-        for (size_t i = 0; i < _ffmpeg->video_streams.size(); i++)
+        for (size_t i = 0; i < _ffmpeg->video_decode_threads.size(); i++)
         {
             _ffmpeg->video_decode_threads[i].finish();
         }
-        for (size_t i = 0; i < _ffmpeg->audio_streams.size(); i++)
+        for (size_t i = 0; i < _ffmpeg->audio_decode_threads.size(); i++)
         {
             _ffmpeg->audio_decode_threads[i].finish();
         }
+        for (size_t i = 0; i < _ffmpeg->subtitle_decode_threads.size(); i++)
+        {
+            _ffmpeg->subtitle_decode_threads[i].finish();
+        }
         // Stop reading packets
         _ffmpeg->reader->finish();
     }
@@ -1347,7 +1750,7 @@ void media_object::close()
         if (_ffmpeg->video_packet_queues[i].size() > 0)
         {
             msg::dbg(_url + ": " + str::from(_ffmpeg->video_packet_queues[i].size())
-                    + " unprocessed video packets in video stream " + str::from(i));
+                    + " unprocessed packets in video stream " + str::from(i));
         }
         for (size_t j = 0; j < _ffmpeg->video_packet_queues[i].size(); j++)
         {
@@ -1370,7 +1773,7 @@ void media_object::close()
         if (_ffmpeg->audio_packet_queues[i].size() > 0)
         {
             msg::dbg(_url + ": " + str::from(_ffmpeg->audio_packet_queues[i].size())
-                    + " unprocessed audio packets in audio stream " + str::from(i));
+                    + " unprocessed packets in audio stream " + str::from(i));
         }
         for (size_t j = 0; j < _ffmpeg->audio_packet_queues[i].size(); j++)
         {
@@ -1381,6 +1784,25 @@ void media_object::close()
     {
         av_free(_ffmpeg->audio_tmpbufs[i]);
     }
+    for (size_t i = 0; i < _ffmpeg->subtitle_codec_ctxs.size(); i++)
+    {
+        if (i < _ffmpeg->subtitle_codecs.size() && _ffmpeg->subtitle_codecs[i])
+        {
+            avcodec_close(_ffmpeg->subtitle_codec_ctxs[i]);
+        }
+    }
+    for (size_t i = 0; i < _ffmpeg->subtitle_packet_queues.size(); i++)
+    {
+        if (_ffmpeg->subtitle_packet_queues[i].size() > 0)
+        {
+            msg::dbg(_url + ": " + str::from(_ffmpeg->subtitle_packet_queues[i].size())
+                    + " unprocessed packets in subtitle stream " + str::from(i));
+        }
+        for (size_t j = 0; j < _ffmpeg->subtitle_packet_queues[i].size(); j++)
+        {
+            av_free_packet(&_ffmpeg->subtitle_packet_queues[i][j]);
+        }
+    }
     if (_ffmpeg->format_ctx)
     {
         av_close_input_file(_ffmpeg->format_ctx);
