@@ -4,6 +4,7 @@
  * Copyright (C) 2010-2011
  * Martin Lambers <marlam@marlam.de>
  * Alexey Osipov <lion-simba@pridelands.ru>
+ * Joe <cuchac@email.cz>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,6 +25,9 @@
 #include <vector>
 #include <unistd.h>
 
+#include "gettext.h"
+#define _(string) gettext(string)
+
 #include "exc.h"
 #include "str.h"
 #include "msg.h"
@@ -42,6 +46,7 @@ player_init_data::player_init_data() :
     urls(),
     video_stream(0),
     audio_stream(0),
+    subtitle_stream(-1),
     benchmark(false),
     fullscreen(false),
     center(false),
@@ -65,6 +70,7 @@ void player_init_data::save(std::ostream &os) const
     s11n::save(os, urls);
     s11n::save(os, video_stream);
     s11n::save(os, audio_stream);
+    s11n::save(os, subtitle_stream);
     s11n::save(os, benchmark);
     s11n::save(os, fullscreen);
     s11n::save(os, center);
@@ -85,6 +91,7 @@ void player_init_data::load(std::istream &is)
     s11n::load(is, urls);
     s11n::load(is, video_stream);
     s11n::load(is, audio_stream);
+    s11n::load(is, subtitle_stream);
     s11n::load(is, benchmark);
     s11n::load(is, fullscreen);
     s11n::load(is, center);
@@ -155,7 +162,7 @@ void player::stop_playback()
     if (_video_output)
     {
         _video_output->exit_fullscreen();
-        _video_output->prepare_next_frame(video_frame());
+        _video_output->prepare_next_frame(video_frame(), subtitle_box());
         _video_output->activate_next_frame();
     }
     notify(notification::play, true, false);
@@ -175,7 +182,7 @@ void player::make_master()
 {
     if (global_player)
     {
-        throw exc("Cannot create a second master player.");
+        throw exc(_("Cannot create a second master player."));
     }
     global_player = this;
 }
@@ -192,29 +199,37 @@ void player::open(const player_init_data &init_data)
     _media_input->open(init_data.urls);
     if (_media_input->video_streams() == 0)
     {
-        throw exc("No video streams found.");
+        throw exc(_("No video streams found."));
     }
     if (init_data.stereo_layout_override)
     {
         if (!_media_input->stereo_layout_is_supported(init_data.stereo_layout, init_data.stereo_layout_swap))
         {
-            throw exc("Cannot set requested stereo layout: incompatible media.");
+            throw exc(_("Cannot set requested stereo layout: incompatible media."));
         }
         _media_input->set_stereo_layout(init_data.stereo_layout, init_data.stereo_layout_swap);
     }
     if (_media_input->video_streams() < init_data.video_stream + 1)
     {
-        throw exc(str::asprintf("Video stream %d not found.", init_data.video_stream + 1));
+        throw exc(str::asprintf(_("Video stream %d not found."), init_data.video_stream + 1));
     }
     _media_input->select_video_stream(init_data.video_stream);
     if (_media_input->audio_streams() > 0 && _media_input->audio_streams() < init_data.audio_stream + 1)
     {
-        throw exc(str::asprintf("Audio stream %d not found.", init_data.audio_stream + 1));
+        throw exc(str::asprintf(_("Audio stream %d not found."), init_data.audio_stream + 1));
     }
     if (_media_input->audio_streams() > 0)
     {
         _media_input->select_audio_stream(init_data.audio_stream);
     }
+    if (_media_input->subtitle_streams() > 0 && _media_input->subtitle_streams() < init_data.subtitle_stream + 1)
+    {
+        throw exc(str::asprintf(_("Subtitle stream %d not found."), init_data.subtitle_stream + 1));
+    }
+    if (_media_input->subtitle_streams() > 0 && init_data.subtitle_stream >= 0)
+    {
+        _media_input->select_subtitle_stream(init_data.subtitle_stream);
+    }
 
     // Create audio output
     if (_media_input->audio_streams() > 0 && !_benchmark)
@@ -279,6 +294,16 @@ void player::open(const player_init_data &init_data)
     }
 }
 
+void player::set_current_subtitle_box()
+{
+    _current_subtitle_box = subtitle_box();
+    if (_next_subtitle_box.is_valid()
+            && _next_subtitle_box.presentation_start_time < _video_pos + _media_input->video_frame_duration())
+    {
+        _current_subtitle_box = _next_subtitle_box;
+    }
+}
+
 int64_t player::step(bool *more_steps, int64_t *seek_to, bool *prep_frame, bool *drop_frame, bool *display_frame)
 {
     *more_steps = false;
@@ -304,6 +329,21 @@ int64_t player::step(bool *more_steps, int64_t *seek_to, bool *prep_frame, bool
             return 0;
         }
         _video_pos = _video_frame.presentation_time;
+        if (_media_input->selected_subtitle_stream() >= 0)
+        {
+            do
+            {
+                _media_input->start_subtitle_box_read();
+                _next_subtitle_box = _media_input->finish_subtitle_box_read();
+                if (!_next_subtitle_box.is_valid())
+                {
+                    msg::dbg("Empty subtitle stream.");
+                    stop_playback();
+                    return 0;
+                }
+            }
+            while (_next_subtitle_box.presentation_stop_time < _video_pos);
+        }
         if (_audio_output)
         {
             _media_input->start_audio_blob_read(_audio_output->required_initial_data_size());
@@ -342,6 +382,7 @@ int64_t player::step(bool *more_steps, int64_t *seek_to, bool *prep_frame, bool
             _first_frame = true;
             *more_steps = true;
             *prep_frame = true;
+            set_current_subtitle_box();
             return 0;
         }
     }
@@ -380,16 +421,33 @@ int64_t player::step(bool *more_steps, int64_t *seek_to, bool *prep_frame, bool
         _seek_request = 0;
         _set_pos_request = -1.0f;
         _media_input->seek(*seek_to);
+        _next_subtitle_box = subtitle_box();
+        _current_subtitle_box = subtitle_box();
 
         _media_input->start_video_frame_read();
         _video_frame = _media_input->finish_video_frame_read();
         if (!_video_frame.is_valid())
         {
-            msg::wrn("Seeked to end of video?!");
+            msg::dbg("Seeked to end of video?!");
             stop_playback();
             return 0;
         }
         _video_pos = _video_frame.presentation_time;
+        if (_media_input->selected_subtitle_stream() >= 0)
+        {
+            do
+            {
+                _media_input->start_subtitle_box_read();
+                _next_subtitle_box = _media_input->finish_subtitle_box_read();
+                if (!_next_subtitle_box.is_valid())
+                {
+                    msg::dbg("Seeked to end of subtitle?!");
+                    stop_playback();
+                    return 0;
+                }
+            }
+            while (_next_subtitle_box.presentation_stop_time < _video_pos);
+        }
         if (_audio_output)
         {
             _audio_output->stop();
@@ -397,7 +455,7 @@ int64_t player::step(bool *more_steps, int64_t *seek_to, bool *prep_frame, bool
             audio_blob blob = _media_input->finish_audio_blob_read();
             if (!blob.is_valid())
             {
-                msg::wrn("Seeked to end of audio?!");
+                msg::dbg("Seeked to end of audio?!");
                 stop_playback();
                 return 0;
             }
@@ -420,6 +478,7 @@ int64_t player::step(bool *more_steps, int64_t *seek_to, bool *prep_frame, bool
         _previous_frame_dropped = false;
         *more_steps = true;
         *prep_frame = true;
+        set_current_subtitle_box();
         return 0;
     }
     else if (_pause_request)
@@ -462,6 +521,18 @@ int64_t player::step(bool *more_steps, int64_t *seek_to, bool *prep_frame, bool
             _first_frame = false;
         }
         _video_pos = _video_frame.presentation_time;
+        if (_media_input->selected_subtitle_stream() >= 0)
+        {
+            // TODO: start subtitle reading asynchronously to use benefits of multithreading
+            while (_next_subtitle_box.is_valid()
+                    && _next_subtitle_box.presentation_stop_time < _video_pos)
+            {
+                _media_input->start_subtitle_box_read();
+                _next_subtitle_box = _media_input->finish_subtitle_box_read();
+                // If the box is invalid, we reached the end of the subtitle stream.
+                // Ignore this and let audio/video continue.
+            }
+        }
         if (!_audio_output)
         {
             _master_time_start += (_video_pos - _master_time_pos);
@@ -478,6 +549,7 @@ int64_t player::step(bool *more_steps, int64_t *seek_to, bool *prep_frame, bool
         else if (!_pause_request)
         {
             *prep_frame = true;
+            set_current_subtitle_box();
         }
         *more_steps = true;
         return 0;
@@ -543,7 +615,7 @@ int64_t player::step(bool *more_steps, int64_t *seek_to, bool *prep_frame, bool
             _drop_next_frame = false;
             if (_master_time_current - _video_pos > _media_input->video_frame_duration() * 75 / 100 && !_benchmark)
             {
-                msg::wrn("Video: delay %g seconds; dropping next frame.", (_master_time_current - _video_pos) / 1e6f);
+                msg::wrn(_("Video: delay %g seconds; dropping next frame."), (_master_time_current - _video_pos) / 1e6f);
                 _drop_next_frame = true;
             }
             if (!_previous_frame_dropped)
@@ -555,7 +627,7 @@ int64_t player::step(bool *more_steps, int64_t *seek_to, bool *prep_frame, bool
                     if (_frames_shown == 100)   //show fps each 100 frames
                     {
                         int64_t now = timer::get_microseconds(timer::monotonic);
-                        msg::inf("FPS: %.2f", static_cast<float>(_frames_shown) / ((now - _fps_mark_time) / 1e6f));
+                        msg::inf(_("FPS: %.2f"), static_cast<float>(_frames_shown) / ((now - _fps_mark_time) / 1e6f));
                         _fps_mark_time = now;
                         _frames_shown = 0;
                     }
@@ -600,7 +672,7 @@ bool player::run_step()
     }
     if (prep_frame)
     {
-        _video_output->prepare_next_frame(_video_frame);
+        _video_output->prepare_next_frame(_video_frame, _current_subtitle_box);
     }
     else if (drop_frame)
     {
@@ -736,6 +808,38 @@ void player::receive_cmd(const command &cmd)
             }
         }
         break;
+    case command::cycle_subtitle_stream:
+        if (_media_input->subtitle_streams() > 1)
+        {
+            int oldstream = _media_input->selected_subtitle_stream();
+            int newstream = oldstream + 1;
+            if (newstream >= _media_input->subtitle_streams())
+            {
+                newstream = -1;
+            }
+            _media_input->select_subtitle_stream(newstream);
+            notify(notification::subtitle_stream, oldstream, newstream);
+            _seek_request = -1;         // Get position right
+        }
+        break;
+    case command::set_subtitle_stream:
+        if (_media_input->subtitle_streams() > 0)
+        {
+            int oldstream = _media_input->selected_subtitle_stream();
+            int newstream;
+            s11n::load(p, newstream);
+            if (newstream < -1 || newstream >= _media_input->subtitle_streams())
+            {
+                newstream = -1;
+            }
+            if (newstream != oldstream)
+            {
+                _media_input->select_subtitle_stream(newstream);
+                notify(notification::subtitle_stream, oldstream, newstream);
+                _seek_request = -1;     // Get position right
+            }
+        }
+        break;
     case command::set_stereo_layout:
         {
             int stereo_layout;
@@ -835,16 +939,6 @@ void player::receive_cmd(const command &cmd)
         parameters_changed = true;
         notify(notification::saturation, oldval, _params.saturation);
         break;
-    case command::seek:
-        s11n::load(p, param);
-        _seek_request = param * 1e6f;
-        /* notify when request is fulfilled */
-        break;
-    case command::set_pos:
-        s11n::load(p, param);
-        _set_pos_request = param;
-        /* notify when request is fulfilled */
-        break;
     case command::adjust_parallax:
         s11n::load(p, param);
         oldval = _params.parallax;
@@ -893,6 +987,69 @@ void player::receive_cmd(const command &cmd)
         parameters_changed = true;
         notify(notification::ghostbust, oldval, _params.ghostbust);
         break;
+    case command::set_subtitle_encoding:
+        {
+            std::string oldenc = _params.subtitle_encoding;
+            s11n::load(p, _params.subtitle_encoding);
+            parameters_changed = true;
+            notify(notification::subtitle_encoding, oldenc, _params.subtitle_encoding);
+            break;
+        }
+    case command::set_subtitle_font:
+        {
+            std::string oldfont = _params.subtitle_font;
+            s11n::load(p, _params.subtitle_font);
+            parameters_changed = true;
+            notify(notification::subtitle_font, oldfont, _params.subtitle_font);
+            break;
+        }
+    case command::set_subtitle_size:
+        {
+            int oldval = _params.subtitle_size;
+            s11n::load(p, _params.subtitle_size);
+            parameters_changed = true;
+            notify(notification::subtitle_size, oldval, _params.subtitle_size);
+            break;
+        }
+    case command::set_subtitle_scale:
+        s11n::load(p, param);
+        oldval = _params.subtitle_scale;
+        _params.subtitle_scale = std::max(param, 0.0f);
+        parameters_changed = true;
+        notify(notification::subtitle_scale, oldval, _params.subtitle_scale);
+        break;
+    case command::set_subtitle_color:
+        {
+            uint64_t oldval = _params.subtitle_color;
+            s11n::load(p, _params.subtitle_color);
+            parameters_changed = true;
+            notify(notification::subtitle_color, oldval, _params.subtitle_color);
+            break;
+        }
+    case command::adjust_subtitle_parallax:
+        s11n::load(p, param);
+        oldval = _params.subtitle_parallax;
+        _params.subtitle_parallax = std::max(std::min(_params.subtitle_parallax + param, 1.0f), -1.0f);
+        parameters_changed = true;
+        notify(notification::parallax, oldval, _params.subtitle_parallax);
+        break;
+    case command::set_subtitle_parallax:
+        s11n::load(p, param);
+        oldval = _params.subtitle_parallax;
+        _params.subtitle_parallax = std::max(std::min(param, 1.0f), -1.0f);
+        parameters_changed = true;
+        notify(notification::subtitle_parallax, oldval, _params.subtitle_parallax);
+        break;
+    case command::seek:
+        s11n::load(p, param);
+        _seek_request = param * 1e6f;
+        /* notify when request is fulfilled */
+        break;
+    case command::set_pos:
+        s11n::load(p, param);
+        _set_pos_request = param;
+        /* notify when request is fulfilled */
+        break;
     }
 
     if (parameters_changed && _video_output)
