@@ -23,9 +23,13 @@
 
 #include <sstream>
 #include <cmath>
+#include <cstdlib>
 
 #include <eq/eq.h>
 
+#include "gettext.h"
+#define _(string) gettext(string)
+
 #include "dbg.h"
 #include "msg.h"
 #include "s11n.h"
@@ -63,7 +67,6 @@ class player_eq_node : public player
 {
 private:
     bool _is_master;
-    bool _first_step;
 
 protected:
     video_output *create_video_output()
@@ -79,7 +82,7 @@ protected:
     }
 
 public:
-    player_eq_node() : player(player::slave), _is_master(false), _first_step(true)
+    player_eq_node() : player(player::slave), _is_master(false)
     {
     }
 
@@ -111,54 +114,62 @@ public:
         start_frame_read();
     }
 
-    void start_frame_read()     // Only called on slave nodes
+    void start_frame_read()
     {
+        // Only called on slave nodes
         get_media_input_nonconst().start_video_frame_read();
     }
 
-    void finish_frame_read()    // Only called on slave nodes
+    void finish_frame_read()
     {
+        // Only called on slave nodes
         _video_frame = get_media_input_nonconst().finish_video_frame_read();
         if (!_video_frame.is_valid())
         {
-            msg::err("Reading input frame failed.");
+            msg::err(_("Reading input frame failed."));
             abort();
         }
     }
 
-    void prepare_next_frame(video_output *vo)
-    {
-        vo->prepare_next_frame(_video_frame);
-    }
-
     void step(bool *more_steps, int64_t *seek_to, bool *prep_frame, bool *drop_frame, bool *display_frame)
     {
-        if (!_is_master && _first_step)
-        {
-            // The master player reads a video frame; do the same on slave players to keep sync
-            start_frame_read();
-            _first_step = false;
-        }
+        // Only called on the master node
         player::step(more_steps, seek_to, prep_frame, drop_frame, display_frame);
     }
+
+    const video_frame &get_video_frame()
+    {
+        return _video_frame;
+    }
+
+    const subtitle_box &get_subtitle()
+    {
+        return _current_subtitle_box;
+    }
 };
 
 /*
- * video_output_eq_window
+ * video_output_eq_channel
  *
- * Implementation of video_output for eq_window.
+ * Implementation of video_output for eq_channel.
  *
  * Much of the video_output interface is not relevant for Equalizer, and thus
  * is implemented with simple stub functions.
  */
 
-class video_output_eq_window : public video_output
+class video_output_eq_channel : public video_output
 {
 private:
-    eq::Window *_wnd;
+    eq::Channel *_channel;
+    const float _canvas_width;
+    const float _canvas_height;
+    const float _canvas_video_area_w;
+    const float _canvas_video_area_h;
 
 protected:
-    void make_context_current() { _wnd->makeCurrent(); }
+    int video_display_width() { return screen_width() * _canvas_video_area_w; }
+    int video_display_height() { return screen_height() * _canvas_video_area_h; }
+    void make_context_current() { _channel->getWindow()->makeCurrent(); }
     bool context_is_stereo() { return false; }
     void recreate_context(bool) { }
     void trigger_update() { }
@@ -166,12 +177,16 @@ protected:
 
 public:
     bool supports_stereo() const { return false; }
-    int screen_width() { return 0; }
-    int screen_height() { return 0; }
-    float screen_aspect_ratio() { return 0.0f; }
-    int width() { return 0; }
-    int height() { return 0; }
-    float aspect_ratio() { return 0.0f; }
+    int screen_width() { return _channel->getPixelViewport().w / _channel->getViewport().w; }
+    int screen_height() { return _channel->getPixelViewport().h / _channel->getViewport().h; }
+    float screen_pixel_aspect_ratio()
+    {
+        float pixels_per_unit_x = _channel->getPixelViewport().w / (_canvas_width * _channel->getViewport().w);
+        float pixels_per_unit_y = _channel->getPixelViewport().h / (_canvas_height * _channel->getViewport().h);
+        return pixels_per_unit_y / pixels_per_unit_x;
+    }
+    int width() { return _channel->getPixelViewport().w; }
+    int height() { return _channel->getPixelViewport().h; }
     int pos_x() { return 0; }
     int pos_y() { return 0; }
     void center() { }
@@ -183,14 +198,25 @@ public:
     void receive_notification(const notification &) { }
 
 public:
-    video_output_eq_window(eq::Window *wnd) : video_output(false), _wnd(wnd)
+    video_output_eq_channel(eq::Channel *channel,
+            float canvas_width, float canvas_height,
+            float canvas_video_area_w, float canvas_video_area_h) :
+        video_output(false),
+        _channel(channel),
+        _canvas_width(canvas_width),
+        _canvas_height(canvas_height),
+        _canvas_video_area_w(canvas_video_area_w),
+        _canvas_video_area_h(canvas_video_area_h)
     {
     }
 
     void display_current_frame(bool mono_right_instead_of_left,
             float x, float y, float w, float h, const GLint viewport[4])
     {
-        video_output::display_current_frame(mono_right_instead_of_left, x, y, w, h, viewport);
+        GLint vp[2][4];
+        std::memcpy(vp[0], viewport, 4 * sizeof(int));
+        std::memcpy(vp[1], viewport, 4 * sizeof(int));
+        video_output::display_current_frame(true, mono_right_instead_of_left, x, y, w, h, vp);
     }
 };
 
@@ -214,10 +240,10 @@ struct ErrorData
 
 static ErrorData errors[] =
 {
-    { ERROR_MAP_FRAMEDATA_FAILED, "Init data mapping failed" },
-    { ERROR_MAP_INITDATA_FAILED, "Frame data mapping failed" },
-    { ERROR_PLAYER_INIT_FAILED, "Video player initialization failed" },
-    { ERROR_OPENGL_2_1_NEEDED, "Need at least OpenGL 2.1" },
+    { ERROR_MAP_FRAMEDATA_FAILED, _("Init data mapping failed.") },
+    { ERROR_MAP_INITDATA_FAILED, _("Frame data mapping failed.") },
+    { ERROR_PLAYER_INIT_FAILED, _("Video player initialization failed.") },
+    { ERROR_OPENGL_2_1_NEEDED, _("Need at least OpenGL 2.1.") },
     { 0, "" } // last!
 };
 
@@ -249,6 +275,8 @@ public:
     eq::uint128_t frame_data_id;
     player_init_data init_data;
     bool flat_screen;
+    float canvas_width;
+    float canvas_height;
     struct { float x, y, w, h, d; } canvas_video_area;
 
     eq_init_data() : init_data()
@@ -278,6 +306,8 @@ protected:
         s11n::save(oss, frame_data_id.low());
         s11n::save(oss, init_data);
         s11n::save(oss, flat_screen);
+        s11n::save(oss, canvas_width);
+        s11n::save(oss, canvas_height);
         s11n::save(oss, canvas_video_area.x);
         s11n::save(oss, canvas_video_area.y);
         s11n::save(oss, canvas_video_area.w);
@@ -295,6 +325,8 @@ protected:
         s11n::load(iss, frame_data_id.low());
         s11n::load(iss, init_data);
         s11n::load(iss, flat_screen);
+        s11n::load(iss, canvas_width);
+        s11n::load(iss, canvas_height);
         s11n::load(iss, canvas_video_area.x);
         s11n::load(iss, canvas_video_area.y);
         s11n::load(iss, canvas_video_area.w);
@@ -310,6 +342,7 @@ protected:
 class eq_frame_data : public co::Object
 {
 public:
+    subtitle_box subtitle;
     parameters params;
     int64_t seek_to;
     bool prep_frame;
@@ -318,8 +351,12 @@ public:
 
 public:
     eq_frame_data() :
-        params(), seek_to(0),
-        prep_frame(false), drop_frame(false), display_frame(false)
+        subtitle(),
+        params(),
+        seek_to(0),
+        prep_frame(false),
+        drop_frame(false),
+        display_frame(false)
     {
     }
 
@@ -332,6 +369,7 @@ protected:
     virtual void getInstanceData(co::DataOStream &os)
     {
         std::ostringstream oss;
+        s11n::save(oss, subtitle);
         s11n::save(oss, params);
         s11n::save(oss, seek_to);
         s11n::save(oss, prep_frame);
@@ -345,6 +383,7 @@ protected:
         std::string s;
         is >> s;
         std::istringstream iss(s);
+        s11n::load(iss, subtitle);
         s11n::load(iss, params);
         s11n::load(iss, seek_to);
         s11n::load(iss, prep_frame);
@@ -405,12 +444,14 @@ public:
         // Find region of canvas to use, depending on the video aspect ratio
         if (getCanvases().size() < 1)
         {
-            msg::err("No canvas in Equalizer configuration.");
+            msg::err(_("No canvas in Equalizer configuration."));
             return false;
         }
         float canvas_w = getCanvases()[0]->getWall().getWidth();
         float canvas_h = getCanvases()[0]->getWall().getHeight();
         float canvas_aspect_ratio = canvas_w / canvas_h;
+        _eq_init_data.canvas_width = canvas_w;
+        _eq_init_data.canvas_height = canvas_h;
         _eq_init_data.canvas_video_area.w = 1.0f;
         _eq_init_data.canvas_video_area.h = 1.0f;
 
@@ -437,9 +478,10 @@ public:
             _eq_init_data.canvas_video_area.x = -0.5f * _eq_init_data.canvas_video_area.w;
             _eq_init_data.canvas_video_area.y = -0.5f * _eq_init_data.canvas_video_area.h;
         }
-        msg::inf("Equalizer canvas:");
-        msg::inf("    %gx%g, aspect ratio %g:1", canvas_w, canvas_h, canvas_w / canvas_h);
-        msg::inf("    Area for %g:1 video: [ %g %g %g %g @ %g ]", frame_template.aspect_ratio,
+        msg::inf(_("Equalizer canvas:"));
+        msg::inf(4, _("%gx%g, aspect ratio %g:1"), canvas_w, canvas_h, canvas_w / canvas_h);
+        msg::inf(4, _("Area for %g:1 video: [ %g %g %g %g @ %g ]"),
+                frame_template.aspect_ratio,
                 _eq_init_data.canvas_video_area.x, _eq_init_data.canvas_video_area.y,
                 _eq_init_data.canvas_video_area.w, _eq_init_data.canvas_video_area.h,
                 _eq_init_data.canvas_video_area.d);
@@ -475,6 +517,7 @@ public:
             this->exit();
         }
         // Update the video state for all (it might have changed via handleEvent())
+        _eq_frame_data.subtitle = _player.get_subtitle();
         _eq_frame_data.params = _player.get_parameters();
         // Commit the updated frame data
         const eq::uint128_t version = _eq_frame_data.commit();
@@ -495,7 +538,7 @@ public:
             case 'q':
                 _controller.send_cmd(command::toggle_play);
                 break;
-            case 's':
+            case 'e':
                 _controller.send_cmd(command::toggle_stereo_mode_swap);
                 break;
             case 'f':
@@ -520,6 +563,12 @@ public:
                  * the change via frame data. */
                 //_controller.send_cmd(command::cycle_audio_stream);
                 break;
+            case 's':
+                /* TODO: cycling subtitle streams is currently not supported with Equalizer.
+                 * We would have to cycle the streams in all node players, and thus communicate
+                 * the change via frame data. */
+                //_controller.send_cmd(command::cycle_subtitle_stream);
+                break;
             case '1':
                 _controller.send_cmd(command::adjust_contrast, -0.05f);
                 break;
@@ -576,6 +625,16 @@ public:
                 break;
             }
         }
+        else if (event->data.type == eq::Event::CHANNEL_POINTER_BUTTON_RELEASE)
+        {
+            float event_px = event->data.pointerButtonRelease.x;        // Event position in pixels
+            float channel_pw = event->data.context.pvp.w;               // Channel width in pixels
+            float event_x = event_px / channel_pw;                      // Event position relative to channel
+            // Event position relative to destination view (which seems to be the same as the canvas?)
+            float dest = event->data.context.vp.x + event_x * event->data.context.vp.w;
+            dest = std::min(std::max(dest, 0.0f), 1.0f);                // Clamp to [0,1] - just to be sure
+            _controller.send_cmd(command::set_pos, dest);               // Seek to this position
+        }
         return true;
     }
 
@@ -786,16 +845,16 @@ protected:
     }
 
 public:
-    void prepare_next_frame(video_output *vo)
+    const video_frame &get_video_frame()
     {
         if (_is_app_node)
         {
             eq_config *config = static_cast<eq_config *>(getConfig());
-            config->player()->prepare_next_frame(vo);
+            return config->player()->get_video_frame();
         }
         else
         {
-            _player.prepare_next_frame(vo);
+            return _player.get_video_frame();
         }
     }
 };
@@ -818,19 +877,11 @@ public:
 
 class eq_window : public eq::Window
 {
-private:
-    video_output_eq_window _video_output;
-
 public:
-    eq_window(eq::Pipe *parent) : eq::Window(parent), _video_output(this)
+    eq_window(eq::Pipe *parent) : eq::Window(parent)
     {
     }
 
-    void display(bool mono_right_instead_of_left, float x, float y, float w, float h, const int viewport[4])
-    {
-        _video_output.display_current_frame(mono_right_instead_of_left, x, y, w, h, viewport);
-    }
-
 protected:
     virtual bool configInitGL(const eq::uint128_t &init_id)
     {
@@ -842,7 +893,7 @@ protected:
         if (!glewContextIsSupported(const_cast<GLEWContext *>(glewGetContext()),
                     "GL_VERSION_2_1 GL_EXT_framebuffer_object"))
         {
-            msg::err("This OpenGL implementation does not support OpenGL 2.1 and framebuffer objects.");
+            msg::err(_("This OpenGL implementation does not support OpenGL 2.1 and framebuffer objects."));
             setError(ERROR_OPENGL_2_1_NEEDED);
             return false;
         }
@@ -856,33 +907,9 @@ protected:
 
     virtual bool configExitGL()
     {
-        msg::dbg(HERE);
-        _video_output.deinit();
         msg::dbg(HERE);
         return eq::Window::configExitGL();
     }
-
-    virtual void frameStart(const eq::uint128_t &, const uint32_t frame_number)
-    {
-        // Get frame data via from the node
-        eq_node *node = static_cast<eq_node *>(getNode());
-        _video_output.set_parameters(node->frame_data.params);
-        // Do as we're told
-        if (node->frame_data.prep_frame)
-        {
-            node->prepare_next_frame(&_video_output);
-        }
-        if (node->frame_data.display_frame)
-        {
-            _video_output.activate_next_frame();
-        }
-        startFrame(frame_number);
-    }
-
-    virtual void frameFinish(const eq::uint128_t &, const uint32_t frame_number)
-    {
-        releaseFrame(frame_number);
-    }
 };
 
 /*
@@ -892,13 +919,30 @@ protected:
 class eq_channel : public eq::Channel
 {
 private:
+    video_output_eq_channel _video_output;
 
 public:
-    eq_channel(eq::Window *parent) : eq::Channel(parent)
+    eq_channel(eq::Window *parent) :
+        eq::Channel(parent),
+        _video_output(this,
+                static_cast<eq_node *>(getNode())->init_data.canvas_width,
+                static_cast<eq_node *>(getNode())->init_data.canvas_height,
+                static_cast<eq_node *>(getNode())->init_data.canvas_video_area.w,
+                static_cast<eq_node *>(getNode())->init_data.canvas_video_area.h)
     {
     }
 
 protected:
+
+    virtual bool configExit()
+    {
+        msg::dbg(HERE);
+        getWindow()->makeCurrent();
+        _video_output.deinit();
+        msg::dbg(HERE);
+        return eq::Channel::configExit();
+    }
+
     virtual void frameDraw(const eq::uint128_t &frame_id)
     {
         // Let Equalizer initialize some stuff
@@ -939,11 +983,34 @@ protected:
         // Display
         glEnable(GL_TEXTURE_2D);
         glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
-        eq_window *window = static_cast<eq_window *>(getWindow());
         bool mono_right_instead_of_left = (getEye() == eq::EYE_RIGHT);
         GLint viewport[4];
         glGetIntegerv(GL_VIEWPORT, viewport);
-        window->display(mono_right_instead_of_left, quad_x, quad_y, quad_w, quad_h, viewport);
+        _video_output.display_current_frame(mono_right_instead_of_left, quad_x, quad_y, quad_w, quad_h, viewport);
+    }
+
+    virtual void frameStart(const eq::uint128_t &, const uint32_t frame_number)
+    {
+        // Get frame data via from the node
+        eq_node *node = static_cast<eq_node *>(getNode());
+        _video_output.set_parameters(node->frame_data.params);
+        // Do as we're told
+        if (node->frame_data.prep_frame)
+        {
+            getWindow()->makeCurrent();
+            _video_output.prepare_next_frame(node->get_video_frame(), node->frame_data.subtitle);
+        }
+        if (node->frame_data.display_frame)
+        {
+            getWindow()->makeCurrent();
+            _video_output.activate_next_frame();
+        }
+        startFrame(frame_number);
+    }
+
+    virtual void frameFinish(const eq::uint128_t &, const uint32_t frame_number)
+    {
+        releaseFrame(frame_number);
     }
 };
 
@@ -992,7 +1059,7 @@ player_equalizer::player_equalizer(int *argc, char *argv[], bool flat_screen) :
     _node_factory = static_cast<void *>(new eq_node_factory);
     if (!eq::init(*argc, argv, static_cast<eq::NodeFactory *>(_node_factory)))
     {
-        throw exc("Equalizer initialization failed.");
+        throw exc(_("Equalizer initialization failed."));
     }
     /* Get a configuration */
     _config = static_cast<void *>(eq::getConfig(*argc, argv));
@@ -1000,7 +1067,7 @@ player_equalizer::player_equalizer(int *argc, char *argv[], bool flat_screen) :
     // eq::getConfig() does not return on other nodes.
     if (!_config)
     {
-        throw exc("Cannot get equalizer configuration.");
+        throw exc(_("Cannot get equalizer configuration."));
     }
 }
 
@@ -1014,7 +1081,7 @@ void player_equalizer::open(const player_init_data &init_data)
     eq_config *config = static_cast<eq_config *>(_config);
     if (!config->init(init_data, _flat_screen))
     {
-        throw exc("Equalizer configuration initialization failed.");
+        throw exc(_("Equalizer configuration initialization failed."));
     }
 }
 
