@@ -4,6 +4,7 @@
  * Copyright (C) 2010-2011
  * Martin Lambers <marlam@marlam.de>
  * Frédéric Devernay <Frederic.Devernay@inrialpes.fr>
+ * Joe <cuchac@email.cz>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -30,6 +31,7 @@
 #include <QDoubleSpinBox>
 #include <QSpinBox>
 #include <QCheckBox>
+#include <QFontComboBox>
 #include <QTimer>
 #include <QSettings>
 #include <QDialog>
@@ -73,6 +75,7 @@ private:
     const player_qt_internal *_player;
     QComboBox *_video_combobox;
     QComboBox *_audio_combobox;
+    QComboBox *_subtitle_combobox;
     QComboBox *_input_combobox;
     QComboBox *_output_combobox;
     QCheckBox *_swap_checkbox;
@@ -84,6 +87,7 @@ private:
 private slots:
     void video_changed();
     void audio_changed();
+    void subtitle_changed();
     void input_changed();
     void output_changed();
     void swap_changed();
@@ -96,6 +100,7 @@ public:
 
     int get_video_stream();
     int get_audio_stream();
+    int get_subtitle_stream();
     void get_stereo_layout(video_frame::stereo_layout_t &stereo_layout, bool &stereo_layout_swap);
     void get_stereo_mode(parameters::stereo_mode_t &stereo_mode, bool &stereo_mode_swap);
 
@@ -196,6 +201,42 @@ public:
     virtual void receive_notification(const notification &note);
 };
 
+class subtitle_dialog: public QDialog, public controller
+{
+    Q_OBJECT
+
+private:
+    parameters *_params;
+    bool _lock;
+    QCheckBox *_encoding_checkbox;
+    QComboBox *_encoding_combobox;
+    QCheckBox *_font_checkbox;
+    QFontComboBox *_font_combobox;
+    QCheckBox *_size_checkbox;
+    QSpinBox *_size_spinbox;
+    QCheckBox *_scale_checkbox;
+    QDoubleSpinBox *_scale_spinbox;
+    QCheckBox *_color_checkbox;
+    QPushButton *_color_button;
+    QColor _color;
+
+    QList<QTextCodec *> find_codecs();
+    void set_color_button(uint32_t c);
+
+private slots:
+    void color_button_pressed();
+    void encoding_changed();
+    void font_changed();
+    void size_changed();
+    void scale_changed();
+    void color_changed();
+
+public:
+    subtitle_dialog(parameters *params, QWidget *parent);
+
+    virtual void receive_notification(const notification &note);
+};
+
 class stereoscopic_dialog : public QDialog, public controller
 {
     Q_OBJECT
@@ -204,12 +245,16 @@ private:
     bool _lock;
     QDoubleSpinBox *_p_spinbox;
     QSlider *_p_slider;
+    QDoubleSpinBox *_sp_spinbox;
+    QSlider *_sp_slider;
     QDoubleSpinBox *_g_spinbox;
     QSlider *_g_slider;
 
 private slots:
     void p_slider_changed(int val);
     void p_spinbox_changed(double val);
+    void sp_slider_changed(int val);
+    void sp_spinbox_changed(double val);
     void g_slider_changed(int val);
     void g_spinbox_changed(double val);
 
@@ -231,6 +276,7 @@ private:
     controls_widget *_controls_widget;
     color_dialog *_color_dialog;
     crosstalk_dialog *_crosstalk_dialog;
+    subtitle_dialog *_subtitle_dialog;
     stereoscopic_dialog *_stereoscopic_dialog;
     player_qt_internal *_player;
     QTimer *_timer;
@@ -249,6 +295,7 @@ private slots:
     void file_open_url();
     void preferences_colors();
     void preferences_crosstalk();
+    void preferences_subtitle();
     void preferences_stereoscopic();
     void help_manual();
     void help_website();
