@@ -67,7 +67,8 @@ bool init_qt()
     {
         qInstallMsgHandler(qt_msg_handler);
         qt_app = new QApplication(qt_argc, const_cast<char **>(qt_argv));
-        QTextCodec::setCodecForCStrings(QTextCodec::codecForName("UTF-8"));
+        // The following is necessary for i18n via gettext:
+        QTextCodec::setCodecForCStrings(QTextCodec::codecForLocale());
         QCoreApplication::setOrganizationName("Bino");
         QCoreApplication::setOrganizationDomain("bino.nongnu.org");
         QCoreApplication::setApplicationName(PACKAGE_NAME);
