@@ -4,6 +4,7 @@
  * Copyright (C) 2010-2011
  * Martin Lambers <marlam@marlam.de>
  * Frédéric Devernay <frederic.devernay@inrialpes.fr>
+ * Joe <cuchac@email.cz>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -38,6 +39,9 @@ static GLEWContext* glewGetContext() { return &_glewContext; }
 #include <QMessageBox>
 #include <QPalette>
 
+#include "gettext.h"
+#define _(string) gettext(string)
+
 #include "exc.h"
 #include "msg.h"
 #include "str.h"
@@ -77,9 +81,9 @@ void video_output_qt_widget::paintGL()
     }
     catch (std::exception &e)
     {
-        QMessageBox::critical(this, "Error", e.what());
+        QMessageBox::critical(this, _("Error"), e.what());
         // Disable further output and stop the player
-        _vo->prepare_next_frame(video_frame());
+        _vo->prepare_next_frame(video_frame(), subtitle_box());
         _vo->activate_next_frame();
         _vo->send_cmd(command::toggle_play);
     }
@@ -108,7 +112,7 @@ void video_output_qt_widget::keyPressEvent(QKeyEvent *event)
     case Qt::Key_Q:
         _vo->send_cmd(command::toggle_play);
         break;
-    case Qt::Key_S:
+    case Qt::Key_E:
         _vo->send_cmd(command::toggle_stereo_mode_swap);
         break;
     case Qt::Key_F:
@@ -127,6 +131,9 @@ void video_output_qt_widget::keyPressEvent(QKeyEvent *event)
     case Qt::Key_A:
         _vo->send_cmd(command::cycle_audio_stream);
         break;
+    case Qt::Key_S:
+        _vo->send_cmd(command::cycle_subtitle_stream);
+        break;
     case Qt::Key_1:
         _vo->send_cmd(command::adjust_contrast, -0.05f);
         break;
@@ -248,7 +255,6 @@ video_output_qt::video_output_qt(bool benchmark, video_container_widget *contain
     {
         _container_widget = new video_container_widget(NULL);
     }
-    _format.setAlpha(true);
     _format.setDoubleBuffer(true);
     if (!benchmark)
     {
@@ -282,18 +288,18 @@ void video_output_qt::init()
             GLenum err = glewInit();
             if (err != GLEW_OK)
             {
-                throw exc(std::string("Cannot initialize GLEW: ")
-                        + reinterpret_cast<const char *>(glewGetErrorString(err)));
+                throw exc(str::asprintf(_("Cannot initialize GLEW: %s"),
+                            reinterpret_cast<const char *>(glewGetErrorString(err))));
             }
             if (!glewIsSupported("GL_VERSION_2_1 GL_EXT_framebuffer_object"))
             {
-                throw exc(std::string("This OpenGL implementation does not support "
-                            "OpenGL 2.1 and framebuffer objects."));
+                throw exc(std::string(_("This OpenGL implementation does not support "
+                                "OpenGL 2.1 and framebuffer objects.")));
             }
         }
         catch (std::exception &e)
         {
-            QMessageBox::critical(_widget, "Error", e.what());
+            QMessageBox::critical(_widget, _("Error"), e.what());
             abort();
         }
         video_output::init();
@@ -320,8 +326,7 @@ void video_output_qt::create_widget()
 {
     _widget = new video_output_qt_widget(this, _format, _container_widget);
     QObject::connect(_container_widget, SIGNAL(move_event()), _widget, SLOT(move_event()));
-    if ((_format.alpha() && !_widget->format().alpha())
-            || (_format.doubleBuffer() && !_widget->format().doubleBuffer())
+    if ((_format.doubleBuffer() && !_widget->format().doubleBuffer())
             || (_format.stereo() && !_widget->format().stereo()))
     {
         try
@@ -329,17 +334,17 @@ void video_output_qt::create_widget()
             if (_format.stereo())
             {
                 // Common failure: display does not support quad buffered stereo
-                throw exc("The display does not support OpenGL stereo mode.");
+                throw exc(_("The display does not support OpenGL stereo mode."));
             }
             else
             {
                 // Should never happen
-                throw exc("Cannot set GL context format.");
+                throw exc(_("Cannot set OpenGL context format."));
             }
         }
         catch (std::exception &e)
         {
-            QMessageBox::critical(_widget, "Error", e.what());
+            QMessageBox::critical(_widget, _("Error"), e.what());
             abort();
         }
     }
@@ -384,7 +389,10 @@ void video_output_qt::trigger_resize(int w, int h)
 {
     _container_widget->set_recommended_size(w, h);
     _container_widget->updateGeometry();
-    _container_widget->window()->adjustSize();
+    if (!_container_is_external)
+    {
+        _container_widget->adjustSize();
+    }
 }
 
 void video_output_qt::mouse_set_pos(float dest)
@@ -428,10 +436,19 @@ int video_output_qt::screen_height()
     return QApplication::desktop()->screenGeometry().height();
 }
 
-float video_output_qt::screen_aspect_ratio()
+float video_output_qt::screen_pixel_aspect_ratio()
 {
-    return static_cast<float>(QApplication::desktop()->widthMM())
-        / static_cast<float>(QApplication::desktop()->heightMM());
+    float screen_pixel_ar =
+        static_cast<float>(QApplication::desktop()->logicalDpiY())
+        / static_cast<float>(QApplication::desktop()->logicalDpiX());
+    if (std::fabs(screen_pixel_ar - 1.0f) < 0.03f)
+    {
+        // This screen most probably has square pixels, and the difference to 1.0
+        // is only due to slightly inaccurate measurements and rounding. Force
+        // 1.0 so that the user gets expected results.
+        screen_pixel_ar = 1.0f;
+    }
+    return screen_pixel_ar;
 }
 
 int video_output_qt::width()
@@ -444,12 +461,6 @@ int video_output_qt::height()
     return _widget->height();
 }
 
-float video_output_qt::aspect_ratio()
-{
-    return static_cast<float>(_widget->widthMM())
-        / static_cast<float>(_widget->heightMM());
-}
-
 int video_output_qt::pos_x()
 {
     return _widget->mapToGlobal(QPoint(0, 0)).x();
